#ifndef __DEMO_2BTN_H__
#define __DEMO_2BTN_H__

#include "draw.h"
#include <stdio.h>
#include <stdlib.h>
#include "iqs269.h"

/* Define DEBUG IO Pin */
//For IO debugging define DEBUG
#define DEBUG_IO
#define		DEBUG_PORT			GPIOC
#define 	DEBUG_PIN			LL_GPIO_PIN_0	//PC0

/* Define IO_1 Pin */
//For IO debugging define DEBUG
#define 	HEAT
#define		HEAT_PORT			GPIOA
#define 	HEAT_PIN			LL_GPIO_PIN_1	//PA1

/* Choose a function to debug */
#define DEBUG_DEMO
//#define DEBUG_HALL
//#define DEBUG_BUTTON
//#define DEBUG_BUTTONS
//#define DEBUG_I2C

#define X_MIDDLE  63
#define Y_MIDDLE  31

#define BUTTON_SIZE  20
#define BUTTON_SIZE2  30
#define NO_OF_BUTTONS 8
#define B1	0
#define B2	1
#define B3	2
#define B4	3
#define B5	4
#define B6	5
#define B7	6
#define B8	7

#define COORD_MIN 7
#define COORD_MAX 248

/*** Types ***/
#pragma pack(1)
typedef union
{
	uint8_t modeByte;
	struct {
		bool dyC: 1;
		bool wear: 1;
		uint8_t reservedForPrevious: 4;
	} current;
	struct {
		uint8_t reservedForCurrent: 4;
		bool dyC: 1;
		bool wear: 1;
	} previous;
} Mode_2btn_t;
#pragma pack(4)

typedef enum {
	wear,
	dynC
} mode_e;

typedef enum {
	startup_2btn_begin = (uint8_t) 0x00,			// 0x00
	startup_2btn_NP,
	startup_2btn_LP,
	startup_2btn_ULP,
	startup_2btn_ref,
	startup_2btn_heat,
	startup_2btn_defaultMode_num,
	startup_2btn_defaultMode_block1,
	startup_2btn_defaultMode_block2,
	startup_2btn_WearDelta,
	startup_2btn_dynamicC_start,
	startup_2btn_dynamicC_plus,
	startup_2btn_dynamicC_minus,
	startup_2btn_hall_plus,
	startup_2btn_hall_plus_loop,
	startup_2btn_hall_minus_loop,
	startup_2btn_clearAll,
	startup_2btn_end,
	startup_2btn_done,
} Demo_Startup_2btn_e;

typedef enum {
	demo_2btn_run_startup = (uint8_t) 0x00,			// 0x00
	demo_2btn_run_idle,
	demo_2btn_run_begin,
	demo_2btn_run_hall,
	demo_2btn_run_default_screen,
	demo_2btn_run_wear,
	demo_2btn_run_dynC,
	demo_2btn_run_refdelta,
	demo_2btn_run_pm,
	demo_2btn_run_done,
} Demo_Run_2btn_e;

typedef enum {
	hall_2btn_none = (uint8_t) 0x00,			// 0x00
	hall_2btn_zero,
	hall_2btn_prox,
	hall_2btn_touch,
	hall_2btn_deep,
	hall_2btn_done
} Hall_2btn_e;

typedef enum {
	pm_2btn_none = (uint8_t) 0x00,			// 0x00
	pm_2btn_np,
	pm_2btn_lp,
	pm_2btn_ulp
} PM_2btn_e;

typedef enum {
	b_2btn_none = (uint8_t) 0x00,			// 0x00
	b_2btn_prox,
	b_2btn_touch,
	b_2btn_deep
} Button_2btn_e;

typedef enum {
	no_ref_state,
	ref_state,
	heat_state,
	heat_disabled_state
} Ref_heat_e;

#pragma pack(1)
typedef struct {
	Demo_Startup_2btn_e			startup_2btn_state;
	Demo_Run_2btn_e 				run_2btn_state;
	bool update;
} demo_2btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Hall_2btn_e state;
	int16_t delta;
	int16_t prev_delta;
} hall_2btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	int16_t deltach1;
	int16_t prev_deltach1;
	int16_t deltach4;
	int16_t prev_deltach4;
	int16_t deltach5;
	int16_t prev_deltach5;
} Delta_e;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	PM_2btn_e state;
} pm_2btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Button_2btn_e state;
	uint8_t new_state;
	uint8_t button_number;
	uint8_t xCoord;
	uint8_t yCoord;
} button_2btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	button_2btn_s button[NO_OF_BUTTONS];
} buttons_2btn_s;
#pragma pack(4)

void Configure_demo_2btn(void);
void init_settings_2btn(void);


void doBtnModeFunction_2btn(void);
void doBtn1Function_2btn(void);


void run_demoStartupRoutine_2btn(void);
void run_demo_2btn(void);
void run_2btn_hall(void);
void run_2btn_pm(void);

void reset_demo_2btn(void);
void iqs269_pointer_init_2btn(iqs269_s *iqs269);

void oledDisplayText_2btn(char* text, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color);
void oledDrawButton_2btn(char* numberText, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color, Button_2btn_e state);
void oledClear_2btn(void);
void oledClearButton_2btn(uint8_t xCoord, uint8_t yCoord, uint8_t size, SSD1306_COLOR color);
void oledClearAllButtons_2btn(void);
void oledClearHeat(void);
void oledClearRef(void);
void oledClearRefDelta(void);
void oledClearPM_2btn(void);
void oledClearCoordinate_2btn(void);
void oledBar_Hall(uint8_t x, int16_t delta, int16_t displayDelta);

/******************************2 Button******************************/
void oledClearModeName_2btn(void);
void oledClearWear(void);
void oledClearWearDelta(void);
void oledClearModeSwitch(void);
void run_button_2btn(uint8_t ch_x, uint8_t button_tag, uint8_t x_coord, uint8_t y_coord, char* numberText);
void oledDrawButton2(char* numberText, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color, Button_2btn_e state);
void run_2btn_default(void);
void run_2btn_dynC(void);
void disable_ref(void);
void disable_heat(void);
void touch_state_dynC(uint8_t ch_x, uint8_t xcoord);
void oledClearDynCbasics(uint8_t xcoord);
void oledClearDynCState(uint8_t xcoord);
void oledClearDynCBar(uint8_t xcoord, uint8_t ycoord);
void oledBar2btn(uint8_t x, int16_t delta, int16_t prevdelta);

#endif // __DEMO_2BTN_H__
