/*
 * demo_8btn.h
 *
 *  Created on: 12 Mar 2020
 *      Author: Georg
 */

#ifndef APPLICATION_USER_INC_DEMO_8BTN_H_
#define APPLICATION_USER_INC_DEMO_8BTN_H_

#include "draw.h"
#include <stdio.h>
#include <stdlib.h>

/* Define DEBUG IO Pin */
//For IO debugging define DEBUG
#define 	DEBUG_IO
#define		DEBUG_PORT			GPIOC
#define 	DEBUG_PIN			LL_GPIO_PIN_0	//PC0

/* Choose a function to debug */
//#define DEBUG_DEMO
#define DEBUG_HALL
//#define DEBUG_BUTTON
//#define DEBUG_BUTTONS
//#define DEBUG_I2C

#define X_MIDDLE  63
#define Y_MIDDLE  31

#define BUTTON_SIZE  20
#define NO_OF_BUTTONS 8
#define B1	0
#define B2	1
#define B3	2
#define B4	3
#define B5	4
#define B6	5
#define B7	6
#define B8	7

/* Slider moving average parameters */
#define X_LCL 0
#define X_UCL 255

#define Y_LCL 20
#define Y_UCL 255

#define X_MAX 255
#define Y_MAX 255

/*** Types ***/
#pragma pack(1)
typedef union
{
	uint8_t modeByte;
	struct {
		bool touch: 1;
		bool _8btn: 1;
		bool abscap: 1;
		bool draw: 1;
		uint8_t reservedForPrevious: 4;
	} current;
	struct {
		uint8_t reservedForCurrent: 4;
		bool touch: 1;
		bool _8btn: 1;
		bool abscap: 1;
		bool draw: 1;
	} previous;
} Mode_8btn_t;
#pragma pack(4)

typedef enum {
	startup_begin = (uint8_t) 0x00,			// 0x00
	startup_NP,
	startup_LP,
	startup_ULP,
	startup_FUNC,
	startup_ABS,
	startup_8btn,
	startup_B1,
	startup_B2,
	startup_B3,
	startup_B4,
	startup_gesture_R,
	startup_B8,
	startup_B7,
	startup_B6,
	startup_B5,
	startup_gesture_L,
	startup_tap_B2,
	startup_tap_Rect1,
	startup_tap_Rect2,
	startup_tap_clear,
	startup_tap_tap,
	startup_up_B6,
	startup_up_B2,
	startup_up_clearB6,
	startup_up_clearB2,
	startup_up_up,
	startup_dwn_B3,
	startup_dwn_B7,
	startup_dwn_clearB3,
	startup_dwn_clearB7,
	startup_dwn_dwn,
	startup_hold_clear,
	startup_hold_all,
	startup_hold_hold,
	startup_hall_plus,
	startup_hall_plus_loop,
	startup_hall_minus_loop,
	startup_clearAll,
	startup_end,
	startup_done
} Demo_Startup_8btn_e;

typedef enum {
	demo_run_startup = (uint8_t) 0x00,			// 0x00
	demo_run_idle,
	demo_run_begin,
	demo_run_hall,
	demo_run_gestures,
	demo_run_coordinates,
	demo_run_clearGesture,
	demo_run_buttons,
	demo_run_b1,
	demo_run_b2,
	demo_run_b3,
	demo_run_b4,
	demo_run_b5,
	demo_run_b6,
	demo_run_b7,
	demo_run_b8,
	demo_run_pm,
	demo_run_absCap,
	demo_run_board_check,
	demo_run_done
} Demo_Run_8btn_e;

typedef enum {
	hall_none = (uint8_t) 0x00,			// 0x00
	hall_zero,
	hall_prox,
	hall_touch,
	hall_deep,
	hall_done
} Hall_8btn_e;

typedef enum {
	coordinate_none = (uint8_t) 0x00,			// 0x00
	coordinate_active,
	coordinate_clear
} Coordinate_e;

typedef enum {
	pm_none = (uint8_t) 0x00,			// 0x00
	pm_np,
	pm_lp,
	pm_ulp
} PM_8btn_e;

typedef enum {
	b_none = (uint8_t) 0x00,			// 0x00
	b_prox,
	b_touch,
	b_deep
} Button_8btn_e;

#pragma pack(1)
typedef struct {
	Demo_Startup_8btn_e			startup_state;
	Demo_Run_8btn_e 				run_state;
	bool update;
} demo_8btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Hall_8btn_e state;
	int16_t delta_filter[5];
	int16_t delta;
	int16_t prev_delta;
} hall_8btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	bool clearGesture;
	bool holdCleared;
} gestures_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Coordinate_e state;
	uint8_t xCoord;
	uint8_t yCoord;
	uint8_t draw_xCoord;
	uint8_t draw_yCoord;
	uint8_t x_filter[5];
	uint8_t y_filter[5];
} coordinates_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	PM_8btn_e state;
} pm_8btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Button_8btn_e state;
	uint8_t new_state;
	uint8_t button_number;
	uint8_t xCoord;
	uint8_t yCoord;
} button_8btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	button_8btn_s button[NO_OF_BUTTONS];
} buttons_8btn_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	uint32_t cap0;
} abscap_s;
#pragma pack(4)


void Configure_demo_8btn(void);
void init_settings_8btn(void);

void run_demoStartupRoutine_8btn(void);
void run_demo_8btn(void);
void run_hall(void);
void run_gestures(void);
void run_coordinates(void);
void run_pm(void);
void run_button(uint8_t ch_x, uint8_t ch_y, uint8_t button_tag, char* numberText);
void run_absCap(void);

void doBtnModeFunction_8btn(void);
void doBtn1Function_8btn(void);
void reset_demo_8btn(void);
void iqs269_pointer_init_8btn(iqs269_s *iqs269);

/* Demo specific LCD functions. */
void oledDisplayText(char* text, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color);
void oledDrawButton_8btn(char* numberText, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color, Button_8btn_e state);
void oledClear(void);
void oledClearButton(uint8_t xCoord, uint8_t yCoord, uint8_t size, SSD1306_COLOR color);
void oledClearAllButtons(void);
void oledClearModeName(void);
void oledClearFunc(void);
void oledClearPM(void);
void oledClearGesture(void);
void oledClearAbsCh(void);
void oledClearAbsCap(void);
void oledClearCoordinate(void);
void oledBar(uint8_t x, int16_t delta, int16_t displayDelta);

#endif /* APPLICATION_USER_INC_DEMO_8BTN_H_ */
