#ifndef __DEMO_INTERFACE_H__
#define __DEMO_INTERFACE_H__

#include <stdio.h>
#include <stdlib.h>
#include "main.h"

/* Define BS0 Pin */
#define		BS0_PORT				GPIOA
#define 	BS0_PIN					LL_GPIO_PIN_0	//PA0

/* Define BS1 Pin */
#define		BS1_PORT				GPIOA
#define 	BS1_PIN					LL_GPIO_PIN_4	//PA4

/* Define BS2 Pin */
#define		BS2_PORT				GPIOB
#define 	BS2_PIN					LL_GPIO_PIN_1	//PB1

/* Define IO_1 Pin */
#define		IO_1_PORT				GPIOA
#define 	IO_1_PIN				LL_GPIO_PIN_1	//PA1

/* Define BTN_MODE Pin */
#define		BTN_MODE_PORT			GPIOC
#define 	BTN_MODE_PIN			LL_GPIO_PIN_2	//PC2

/* Define BTN1 Pin */
#define		BTN1_PORT				GPIOC
#define 	BTN1_PIN				LL_GPIO_PIN_3	//PC3

/* Define OLED SSD1306 GPIO Pin */  	// Used to switch screen on or off
#define		OLED_GPIO_PORT			GPIOB
#define 	OLED_GPIO_PIN			LL_GPIO_PIN_3	//PB3

/* Define CT SENSE Pin */
#define		CT_SENSE_PORT			GPIOC
#define 	CT_SENSE_PIN			LL_GPIO_PIN_6	//PC6

//Device Info
#define PRODUCT_NUM         0x4F
#define SW_NUM_V1           0x02
#define SW_NUM_V2           0x03
#define HW_NUM				0x0D

typedef enum {
	board_none = (uint8_t) 0x00,			// 0x00
	board_8btn,
	board_2btn,
	board_slider,
	board_error,
	board_error_iqs,
	board_error_ct
} Board_e;

typedef enum {
	iqs_none = (uint8_t) 0x00,			// 0x00
	iqs_correct,
	iqs_sw_error,
	iqs_hw_error,
	iqs_prod_error
} iqs_board_e;

#pragma pack(1)
typedef union
{
	uint8_t bsByte;
	struct {
		bool _0: 1;
		bool _1: 1;
		bool _2: 1;
		bool prev0: 1;
		bool prev1: 1;
		bool prev2: 1;
		bool _8button: 1;
		bool _2button: 1;
	} bs;
} BoardSelect_t;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Board_e state;
	iqs_board_e iqs_state;
//	uint8_t iqs_sw_num;
//	uint8_t iqs_hw_num;
//	uint8_t iqs_prod_num;
} board_s;
#pragma pack(4)

void Configure_Demo(void);
void run_board_select(void);
void reset_after_insertation(void);

void BTN_MODE_Callback(uint8_t IR);
void BTN1_Callback(uint8_t IR);
void CT_SENSE_Callback(uint8_t IR);

bool readIQSDeviceInfo(void);
void readBoardSelectPins(void);
void check_BTN_MODE(void);
void check_BTN_1(void);
void check_CT_SENSE(void);

#endif // __DEMO_INTERFACE_H__
