#ifndef __DEMO_SLIDER_H__
#define __DEMO_SLIDER_H__

#include "draw.h"
#include <stdio.h>
#include <stdlib.h>
#include "iqs269.h"

/* Define DEBUG IO Pin */
//For IO debugging define DEBUG
#define 	DEBUG_IO
#define		DEBUG_PORT			GPIOC
#define 	DEBUG_PIN			LL_GPIO_PIN_0	//PC0

/* Choose a function to debug */
//#define DEBUG_DEMO
//#define DEBUG_HALL
//#define DEBUG_BUTTON
//#define DEBUG_BUTTONS
//#define DEBUG_I2C

#define X_MIDDLE  63
#define Y_MIDDLE  31

/* Slider moving average parameters */
#define X_LCL 0
#define X_UCL 255

#define Y_LCL 20
#define Y_UCL 255

#define X_MAX 255
#define Y_MAX 255

typedef enum {
	startup_slider_begin = (uint8_t) 0x00,			// 0x00
	startup_slider_mode1_1,
	startup_slider_mode1_2,
	startup_slider_mode2_1,
	startup_slider_mode2_2,
	startup_slider_mode3_1,
	startup_slider_mode3_2,
	startup_slider_clearAll,
	startup_slider_end,
	startup_slider_done
} Demo_Startup_Slider_e;

typedef enum {
	demo_slider_run_startup = (uint8_t) 0x00,			// 0x00
	demo_slider_run_idle,
	demo_slider_run_begin,
	demo_slider_run_gestures,
	demo_slider_run_relative_coordinates,
	demo_slider_run_clearGesture,
	demo_slider_run_volume,
	demo_slider_run_swipe_detect,
	demo_slider_run_swipe_hold,
	demo_slider_run_relative_swipe,
	demo_slider_run_select_mode,
	demo_slider_run_done
} Demo_Run_Slider_e;

typedef enum {
	coordinate_slider_none = (uint8_t) 0x00,			// 0x00
	coordinate_slider_active,
	coordinate_slider_clear
} Coordinate_Slider_e;

#pragma pack(1)
typedef struct {
	Demo_Startup_Slider_e			startup_slider_state;
	Demo_Run_Slider_e 				run_slider_state;
	bool update;
} demo_slider_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	int16_t delta;
	int16_t prev_delta;
	int16_t volumeChange;
	uint8_t absVol;
	uint8_t prev_vol;
} volume_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	bool clearGesture;
} gestures_slider_s;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	Coordinate_Slider_e state;
	uint8_t xCoord;
	uint8_t yCoord;
	uint8_t x_filter[5];
} coordinates_slider_s;
#pragma pack(4)


void Configure_demo_slider(void);
void init_settings_slider(void);
void reset_demo_slider(void);
void iqs269_pointer_init_slider(iqs269_s *iqs269);

void run_demoStartupRoutine_slider(void);
void run_demo_slider(void);
void run_gestures_slider(void);
void run_relative_coordinates(void);
void run_volume(void);
void run_select_mode(void);

void oledDisplayText_slider(char* text, uint8_t xCoord, uint8_t yCoord, FontDef Font, SSD1306_COLOR color);
void oledClear_slider(void);
void oledClearPM_slider(void);
void oledClearGesture_slider(void);
void oledClearMode(void);

void oledVolumeBar(int16_t delta, uint8_t vol);
void init_volume_startpos(void);
void run_swipe_detect(void);
void run_swipe_hold(void);
void run_relative_swipe(void);







#endif // __DEMO_SLIDER_H__
