/*** Includes ***/
#include <demo_interface.h>
#include "main.h"
#include "draw.h"
#include "AZQ_util.h"
#include "iqs269.h"
#include "ftoa.h"
#include "spi_dma.h"
#include "demo_8btn.h"
#include "demo_2btn.h"
#include "demo_slider.h"


board_s board;
BoardSelect_t boardSelect;
iqs269_s iqs269;
// Screen object
SSD1306_t ssd1306;

uint32_t timerBTN_Mode;
uint32_t timerBTN1;
uint32_t timerAti;
uint32_t timerCT_Sense;
uint32_t timerCT_Sense_remove;

bool btnModePressed = false;
bool btn1Pressed = false;
bool ct_Sensed = false;
bool ct_Removed = false;
bool ct_Plugged = false;

/**
 * @brief Configure settings for IQS269 demo
 *
 * @param None
 *
 * @retval None
 */
void Configure_Demo(void){
	/* Pointer initialization. */
	iqs269_pointer_init_8btn(&iqs269);
	iqs269_pointer_init_2btn(&iqs269);
	iqs269_pointer_init_slider(&iqs269);

	/* Assign Parameters to I2C Bus. */
	iqs269.azq_iqs269.i2cBus.i2cPeripheral 	= I2C1;
	iqs269.azq_iqs269.i2cBus.sclPort 		= GPIOB;
	iqs269.azq_iqs269.i2cBus.sclPin 		= LL_GPIO_PIN_8;			//PB8
	iqs269.azq_iqs269.i2cBus.sclMode		= LL_GPIO_MODE_ALTERNATE;
	iqs269.azq_iqs269.i2cBus.scl_AF			= LL_GPIO_AF_6;
	iqs269.azq_iqs269.i2cBus.sdaPort 		= GPIOB;
	iqs269.azq_iqs269.i2cBus.sdaPin 		= LL_GPIO_PIN_9;			//PB9
	iqs269.azq_iqs269.i2cBus.sdaMode		= LL_GPIO_MODE_ALTERNATE;
	iqs269.azq_iqs269.i2cBus.sda_AF			= LL_GPIO_AF_6;
	iqs269.azq_iqs269.i2cBus.i2cTimeout_ms 	= 100;
	iqs269.azq_iqs269.i2cBus.priority 		= 1;
	iqs269.azq_iqs269.i2cBus.i2cMode	 	= I2C_MODE_FAST;
	iqs269.azq_iqs269.i2cBus.GPIOspeed		= LL_GPIO_SPEED_FREQ_LOW;


	/* Setup IQS ready pin. */
	iqs269.azq_iqs269.i2cAddress 		= 0x44;
	iqs269.azq_iqs269.readyPort 		= GPIOA;
	iqs269.azq_iqs269.readyPin 			= LL_GPIO_PIN_10;				//PA10
	iqs269.azq_iqs269.azqTimeout_ms 	= 100;

	/* Assign function to be called when ready interrupt is received. Also define which type of interrupt. */
	iqs269.azq_iqs269._ready_callback = (void *) &iqs269_rdy_callback;
	iqs269.azq_iqs269.Interrupt_Type = LL_EXTI_TRIGGER_FALLING;

	/* Activate falling edge and rising edge interrupts for device. */
	iqs269.azq_iqs269.Falling_IR = true;
	iqs269.azq_iqs269.Rising_IR = true;

	/* Select interrupt priority. */
	iqs269.azq_iqs269.IR_priority = 0;

	init_iqs269(&iqs269);

	ssd1306_Init(&ssd1306);  //pointer init
	init_spi();

	/* Configure OLED GPIO Pin. */
	gpio_init(OLED_GPIO_PORT, OLED_GPIO_PIN, LL_GPIO_MODE_OUTPUT, FUNCTION_NOT_USED, LL_GPIO_OUTPUT_PUSHPULL, LL_GPIO_PULL_UP, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	/* Set OLED GPIO Pin to power the LCD. */
	LL_GPIO_ResetOutputPin(OLED_GPIO_PORT, OLED_GPIO_PIN);

	/* Configure BS0 Pin */
	gpio_init(BS0_PORT, BS0_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_NO, LL_GPIO_SPEED_FREQ_VERY_HIGH);

	/* Configure BS1 Pin */
	gpio_init(BS1_PORT, BS1_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_NO, LL_GPIO_SPEED_FREQ_VERY_HIGH);

	/* Configure BS2 Pin */
	gpio_init(BS2_PORT, BS2_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_NO, LL_GPIO_SPEED_FREQ_VERY_HIGH);

	/* Enable GPIO Clock (to be able to program the configuration registers) */
	Enable_GPIO_Clock(GPIOC);

	/* Configure BTN_MODE */
	/* GPIO Config */
	gpio_init(BTN_MODE_PORT, BTN_MODE_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_UP, LL_GPIO_SPEED_FREQ_LOW);
	/* Configure Interrupt */
	LL_EXTI_EnableIT_0_31(BTN_MODE_PIN);
	Configure_EXTI(BTN_MODE_PORT, BTN_MODE_PIN, 2, true, true, &BTN_MODE_Callback);

	/* Configure BTN1 */
	/* GPIO Config */
	gpio_init(BTN1_PORT, BTN1_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_UP, LL_GPIO_SPEED_FREQ_LOW);
	/* Configure Interrupt */
	LL_EXTI_EnableIT_0_31(BTN1_PIN);
	Configure_EXTI(BTN1_PORT, BTN1_PIN, 2, true, true, &BTN1_Callback);

	/* Configure DEBUG Pin */
	#ifdef DEBUG_IO
	gpio_init(DEBUG_PORT, DEBUG_PIN, LL_GPIO_MODE_OUTPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_UP, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetOutputPin(DEBUG_PORT, DEBUG_PIN);
	#endif

	/* CT SENSE GPIO Config */
	gpio_init(CT_SENSE_PORT, CT_SENSE_PIN, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_UP, LL_GPIO_SPEED_FREQ_LOW);
	/* Configure Interrupt */
	LL_EXTI_EnableIT_0_31(CT_SENSE_PIN);
	Configure_EXTI(CT_SENSE_PORT, CT_SENSE_PIN, 2, true, true, &CT_SENSE_Callback);

	/* Configure board specific demos here. */
	Configure_demo_8btn();
	Configure_demo_2btn();
	Configure_demo_slider();

	/* Initialize board select pins (used for board selection) */
	readBoardSelectPins();

	if (!(LL_GPIO_IsInputPinSet(CT_SENSE_PORT, CT_SENSE_PIN))) {
		ct_Plugged = true;
	}
	if ( (!boardSelect.bs._0) && (boardSelect.bs._1) && (boardSelect.bs._2) )	// 011
	{
		board.state = board_8btn;
		reset_demo_8btn();
	}
	else if((boardSelect.bs._0) && (!boardSelect.bs._1) && (boardSelect.bs._2) )  //101
	{
		board.state = board_2btn;
		reset_demo_2btn();
	}
	else if((boardSelect.bs._0) && (boardSelect.bs._1) && (!boardSelect.bs._2) )  //110
	{
		board.state = board_slider;
		reset_demo_slider();
	}
	else
	{
		board.state = board_error;
	}

	/* Power on LCD. */
	LL_GPIO_SetOutputPin(OLED_GPIO_PORT, OLED_GPIO_PIN);
}

/**
 * @brief Check board select pins to run the appropriate demo. Buffer the last board select and read the board select pins. Display an error if the correct board is not inserted.
 *
 * @param None.
 *
 *	@note Call this function in main run loop.
 *
 * @retval None
 */
void run_board_select(void)
{
	/* Run IQS driver. */
	run_iqs269();
	/* Wait for IQS to finish initialization. */
	if (iqs269.init_state != IQS269_INIT_DONE)
		return;

	/* Run other drivers. */
	run_LCD();
	check_BTN_MODE();
	check_BTN_1();
	check_CT_SENSE();

	uint8_t new_state;

	//Buffer last board select
	boardSelect.bs.prev0 = boardSelect.bs._0;
	boardSelect.bs.prev1 = boardSelect.bs._1;
	boardSelect.bs.prev2 = boardSelect.bs._2;

	//Read BS0,BS1,BS2 (used for board selection)
	readBoardSelectPins();

	if ((readIQSDeviceInfo()) && (board.iqs_state != iqs_none)) {

		if ((ct_Plugged)) {
			new_state = board_error_ct;
			boardSelect.bs._8button = false;
			boardSelect.bs._2button = false;
		}
		else if ( (!boardSelect.bs._0) && (boardSelect.bs._1) && (boardSelect.bs._2) )	//011
		{
			new_state = board_8btn;

			boardSelect.bs._2button = false;
			boardSelect.bs._8button = true;
		}
		else if((boardSelect.bs._0) && (!boardSelect.bs._1) && (boardSelect.bs._2) )	//101
		{
			new_state = board_2btn;

			boardSelect.bs._2button = true;
			boardSelect.bs._8button = false;
		}
		else if((boardSelect.bs._0) && (boardSelect.bs._1) && (!boardSelect.bs._2) )  //110
		{
			new_state = board_slider;

			boardSelect.bs._2button = false;
			boardSelect.bs._8button = false;
		}
		//	else if (mode.current.abscap)		//modes not defined in demo interface
		//	{
		//		/* Continue as normal while in Abs cap mode, although board is removed. */
		//		new_state = board_8btn;
		//	}
		else
		{
			new_state = board_error;
			boardSelect.bs._8button = false;
			boardSelect.bs._2button = false;
		}
	} else if ((board.iqs_state != iqs_none))
	{
		new_state = board_error_iqs;
		boardSelect.bs._8button = false;
		boardSelect.bs._2button = false;
	}

	/* Run the board specific demo*/
	switch (new_state) {
	case board_8btn:
		if (board.state != new_state) {
			board.state = new_state;
			reset_after_insertation();
		}
		/*	Run	8btn demo. */
		run_demo_8btn();
		break;

	case board_2btn:
		if (board.state != new_state) {
			board.state = new_state;
			reset_after_insertation();
		}
		/*	Run	2btn demo. */
		run_demo_2btn();
		break;

	case board_slider:
		if (board.state != new_state) {
			board.state = new_state;
			reset_after_insertation();
		}
		/*	Run volume	slider demo. */
		run_demo_slider();
		break;

	case board_error:
		if((board.state != new_state))		//Remove bs._0 check		(boardSelect.bs._0) &&
		{
			if (ssd1306_CheckScreenReady(0)) {
				board.state = new_state;
				oledClear();
				oledDisplayText("Button", X_MIDDLE-33, 0, Font_11x18, White);
				oledDisplayText("Board", X_MIDDLE-28, 22, Font_11x18, White);
				oledDisplayText("Required!", 15, 46, Font_11x18, White);
			}
		}
		break;
	case board_error_iqs:
		if(board.state != new_state)
		{
			if (ssd1306_CheckScreenReady(0)) {
				board.state = new_state;
				oledClear();
				switch (board.iqs_state) {
				case iqs_prod_error:
					oledDisplayText("!", X_MIDDLE, 5, Font_11x18, White);
					oledDisplayText("Device connected is", 5, 35, Font_6x8, White);
					oledDisplayText("not an IQS269A", 16, 45, Font_6x8, White);
					break;
				case iqs_sw_error:
					oledDisplayText("!", X_MIDDLE, 5, Font_11x18, White);
					oledDisplayText("Outdated IQS269A", 15, 30, Font_6x8, White);
					oledDisplayText("Firmware", 39, 40, Font_6x8, White);
					oledDisplayText("IQS269A V3 or later", 10, 50, Font_6x8, White);
					oledDisplayText("Required", 10, 60, Font_6x8, White);
					break;
				case iqs_hw_error:
					oledDisplayText("!", X_MIDDLE, 5, Font_11x18, White);
					oledDisplayText("Device connected is", 5, 35, Font_6x8, White);
					oledDisplayText("not an IQS269A", 16, 45, Font_6x8, White);
					break;
				default:
					break;
				}
			}
		}
		break;
	case board_error_ct:
		if((board.state != new_state))
		{
			if (ssd1306_CheckScreenReady(0)) {
				board.state = new_state;
				oledClear();
				oledDisplayText("CT210A", X_MIDDLE-33, 0, Font_11x18, White);
				oledDisplayText("Plugged", X_MIDDLE-36, 22, Font_11x18, White);
				oledDisplayText("In!", X_MIDDLE-11, 46, Font_11x18, White);
			}
		}
		break;
	default:
		break;
	}
}

/**
 * @brief Debounce and reset board specific demo after a board was inserted
 *
 * @param None.
 *
 * @retval None.
 */
void reset_after_insertation(void)
{
	/* Debounce before doing resetting IQS269 and demo state machine. */
	if (HAL_CheckTick(timerAti, 300, true)) {
		timerAti = HAL_GetTick();

		switch (board.state) {
		case board_8btn:
			reset_demo_8btn();
			break;
		case board_2btn:
			reset_demo_2btn();
			break;
		case board_slider:
			reset_demo_slider();
			break;
		case board_error_ct:
			if ( (!boardSelect.bs._0) && (boardSelect.bs._1) && (boardSelect.bs._2) )	//011
			{
				reset_demo_8btn();
			}
			else if((boardSelect.bs._0) && (!boardSelect.bs._1) && (boardSelect.bs._2) )	//101
			{
				reset_demo_2btn();
			}
			else if((boardSelect.bs._0) && (boardSelect.bs._1) && (!boardSelect.bs._2) )  //110
			{
				reset_demo_slider();
			}
			break;
		default:
			break;
		}
	}
}

/**
 * @brief Read the 3 board select pins and store in boardSelect.bs._x
 *
 * @param None.
 *
 * @retval None.
 */
void readBoardSelectPins(void)
{
	boardSelect.bs._0 = LL_GPIO_IsInputPinSet(BS0_PORT, BS0_PIN);   //BS[2;1;0] => 110
	boardSelect.bs._1 = LL_GPIO_IsInputPinSet(BS1_PORT, BS1_PIN);   //BS[2;1;0] => 101
	boardSelect.bs._2 = LL_GPIO_IsInputPinSet(BS2_PORT, BS2_PIN);   //BS[2;1;0] => 011
}

/**
 * @brief Read the hardware and software number from the IQS device
 *
 * @param None.
 *
 * @retval None.
 */
bool readIQSDeviceInfo(void)
{
	uint8_t new_state;


	if ((iqs269_getProdNumber() == PRODUCT_NUM) && (iqs269_getSoftVersion() == SW_NUM_V2) && (iqs269_getHardNumber() == HW_NUM)) {
		new_state = iqs_correct;
	}
	else if (iqs269_getHardNumber() != HW_NUM)
	{
		new_state = iqs_hw_error;
	}
	else if (iqs269_getProdNumber() != PRODUCT_NUM)
	{
		new_state = iqs_prod_error;
	}
	else if (iqs269_getSoftVersion() != SW_NUM_V2)
	{
		new_state = iqs_sw_error;
	}

	if (board.iqs_state != new_state) {
		board.iqs_state = new_state;
	}


	if (board.iqs_state == iqs_correct) {
		return true;
	}
	else
	{
		return false;
	}
}

/**
 * @brief Button Mode callback function.
 *
 * @param uint8_t : \b IR - Type of interrupt trigger.
 *
 * @note IR Types: LL_EXTI_TRIGGER_FALLING, LL_EXTI_TRIGGER_RISING or LL_EXTI_TRIGGER_RISING_FALLING
 *
 * @retval None.
 */
void BTN_MODE_Callback(uint8_t IR){
	if(((IR == LL_EXTI_TRIGGER_FALLING) || (IR == LL_EXTI_TRIGGER_RISING_FALLING)) && (!LL_GPIO_IsInputPinSet(BTN_MODE_PORT, BTN_MODE_PIN)))	// if the signal bounces there will be no falling trigger, only rising and falling trigger
	{
		// Debounce
		if(HAL_CheckTick(timerBTN_Mode, 250, true)){		// Button press
			timerBTN_Mode = HAL_GetTick();
			btnModePressed = true;
		}
	}
}

/**
 * @brief Check if button Mode was pressed.
 *
 * @param None.
 *
 * @retval None.
 */
void check_BTN_MODE(void)
{
	if (btnModePressed) {
		btnModePressed = false;

		/* Determine the function of the push button based on the board inserted. */
		switch (board.state) {
		case board_8btn:
			doBtnModeFunction_8btn();
			break;
		case board_2btn:
			doBtnModeFunction_2btn();
			break;
		case board_slider:
			run_select_mode();
			break;
		case board_error:

			break;
		default:
			break;
		}
	}
}



/**
 * @brief Button 1 callback function.
 *
 * @param uint8_t : \b IR - Type of interrupt trigger.
 *
 * @note IR Types: LL_EXTI_TRIGGER_FALLING, LL_EXTI_TRIGGER_RISING or LL_EXTI_TRIGGER_RISING_FALLING
 *
 * @retval None.
 */
void BTN1_Callback(uint8_t IR){
	if(((IR == LL_EXTI_TRIGGER_FALLING) || (IR == LL_EXTI_TRIGGER_RISING_FALLING)) && (!LL_GPIO_IsInputPinSet(BTN1_PORT, BTN1_PIN)))	// if the signal bounces there will be no falling trigger, only rising and falling trigger
	{
		// Debounce
		if(HAL_CheckTick(timerBTN1, 250, true)){		// Button press
			timerBTN1 = HAL_GetTick();
			btn1Pressed = true;
		}
	}
}

/**
 * @brief Check if button 1 was pressed.
 *
 * @param None.
 *
 * @retval None.
 */
void check_BTN_1(void)
{
	if (btn1Pressed) {
		btn1Pressed = false;

		/* Determine the function of the push button based on the board inserted. */
		switch (board.state) {
		case board_8btn:
			doBtn1Function_8btn();
			break;
		case board_2btn:
			doBtn1Function_2btn();
			break;
		case board_error:

			break;
		default:
			break;
		}
	}
}

/**
 * @brief CT SENSE callback function.
 *
 * @param uint8_t : \b IR - Type of interrupt trigger.
 *
 * @note IR Types: LL_EXTI_TRIGGER_FALLING, LL_EXTI_TRIGGER_RISING or LL_EXTI_TRIGGER_RISING_FALLING
 *
 * @retval None.
 */
void CT_SENSE_Callback(uint8_t IR)
{
	/* When CT is plugged in. */
	if(((IR == LL_EXTI_TRIGGER_FALLING) || (IR == LL_EXTI_TRIGGER_RISING_FALLING)) && (!LL_GPIO_IsInputPinSet(CT_SENSE_PORT, CT_SENSE_PIN)))	// if the signal bounces there will be no falling trigger, only rising and falling trigger
	{
		// Debounce
		if(HAL_CheckTick(timerCT_Sense, 1000, true))
		{
			timerCT_Sense = HAL_GetTick();
			ct_Sensed = true;
			ct_Plugged = true;
		}
	}
	/* When CT is removed. */
	if(((IR == LL_EXTI_TRIGGER_RISING) || (IR == LL_EXTI_TRIGGER_RISING_FALLING)) && (LL_GPIO_IsInputPinSet(CT_SENSE_PORT, CT_SENSE_PIN)))	// if the signal bounces there will be no falling trigger, only rising and falling trigger
	{
		// Debounce
		if((HAL_CheckTick(timerCT_Sense_remove, 1000, true)) && (HAL_CheckTick(timerCT_Sense, 1000, true)))
		{
			timerCT_Sense_remove = HAL_GetTick();
			ct_Removed = true;
			ct_Plugged = false;
		}
//		else if ((HAL_CheckTick(timerCT_Sense, 5, false)))
//		{
//			timerCT_Sense_remove = HAL_GetTick();
//			ct_Removed = true;
//			ct_Plugged = false;
//		}
	}
}

/**
 * @brief Check if CT210A is plugged in
 *
 * @param None.
 *
 * @retval None.
 */
void check_CT_SENSE(void)
{
	if (ct_Sensed)
	{
		ct_Sensed = false;

		iqs269_deQueueValueUpdates();
	}

	if (ct_Removed)	// && (HAL_CheckTick(timerCT_Sense_remove, 5, true))
	{
		ct_Removed = false;

//		iqs269_QueueValueUpdates();
		reset_after_insertation();
	}
}

