/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    i2c.c							 				  	 	  	  *
 * @brief 	    i2c driver file								  		  	  	  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    28/03/2017                                                    *
 ******************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "List.h"
#include "AZQ_util.h"
#include <stdbool.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

//I2C_Com_Variables_t  I2C_ComVariables[2]; // Array of Variables needed for I2C communication

List_t	i2c_transactions[2];
I2C_transaction_s current_transaction[2];



uint8_t write_buffer[256] = {0};

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
void Configure_I2C_Master(I2C_HardwareBus_t *i2cBus);

void Start_I2C_Write_Transfer(I2C_TypeDef *I2Cx);
void Start_I2C_Read_Transfer(I2C_TypeDef *I2Cx);
void Transmission_Start_Callback(I2C_TypeDef *I2Cx);
void Transmission_Register_Callback(I2C_TypeDef *I2Cx);
void Transmission_Callback(I2C_TypeDef *I2Cx);	// write function
void Reception_Callback(I2C_TypeDef *I2Cx);     // read function

/**
  * @brief  init i2c function
  * @param  None
  * @retval None
  */
void init_i2c(I2C_HardwareBus_t *i2cBus)
{
	Configure_I2C_Master(i2cBus);

	List_init(&i2c_transactions[0], sizeof(I2C_transaction_s), 20);
	List_init(&i2c_transactions[1], sizeof(I2C_transaction_s), 20);	 //todo max transactions 20

}

/**
  * @brief  This function configures I2C1 in Master mode.
  * @note   This function is used to :
  *         -1- Enables GPIO clock and configures the I2C1 pins.
  *         -2- Enable the I2C1 peripheral clock.
  *         -3- Configure NVIC for I2C1.
  *         -4- Configure I2C1 functional parameters.
  *         -5- Enable I2C1.
  *         -6- Enable I2C1 transfer complete/error interrupts.
  * @note   Peripheral configuration is minimal configuration from reset values.
  *         Thus, some useless LL unitary functions calls below are provided as
  *         commented examples - setting is default configuration from reset.
  * @param  None
  * @retval None
  */
void Configure_I2C_Master(I2C_HardwareBus_t *i2cBus)
{

	  LL_I2C_InitTypeDef I2C_InitStruct = {0};



	  /* Enable GPIO clocks for SCL and SDA */
	  Enable_GPIO_Clock(i2cBus->sclPort);
	  Enable_GPIO_Clock(i2cBus->sdaPort);

	  /**I2C1 GPIO Configuration
	  */
	  /* Configure SCL Pin as : Alternate function, User selected speed, Open drain, Pull up */
	  gpio_init(i2cBus->sclPort, i2cBus->sclPin, i2cBus->sclMode, i2cBus->scl_AF, LL_GPIO_OUTPUT_OPENDRAIN, LL_GPIO_PULL_UP, i2cBus->GPIOspeed);

	  /* Configure SDA Pin as : Alternate function, User selected speed, Open drain, Pull up */
	  gpio_init(i2cBus->sdaPort, i2cBus->sdaPin, i2cBus->sdaMode, i2cBus->sda_AF, LL_GPIO_OUTPUT_OPENDRAIN, LL_GPIO_PULL_UP, i2cBus->GPIOspeed);


	  /* Peripheral clock enable */
	  Enable_I2C_Clock(i2cBus->i2cPeripheral);

	  /* I2C interrupt Init */
	  init_I2C_interrupt(i2cBus->i2cPeripheral, i2cBus->priority);

	  /* USER CODE END I2C_Init 1 */
	  /** I2C Initialization
	  */
	  I2C_InitStruct.PeripheralMode		 = LL_I2C_MODE_I2C;
	  I2C_InitStruct.Timing				 = 0x00501D63;// __LL_I2C_CONVERT_TIMINGS(PRESC(i2cBus->i2cMode), SCLDEL(i2cBus->i2cMode), SDADEL(i2cBus->i2cMode), SCLH(i2cBus->i2cMode), SCLL(i2cBus->i2cMode));
	  I2C_InitStruct.AnalogFilter		 = LL_I2C_ANALOGFILTER_ENABLE;
	  I2C_InitStruct.DigitalFilter		 = 0;
	  I2C_InitStruct.OwnAddress1 		 = 0;
	  I2C_InitStruct.TypeAcknowledge 	 = LL_I2C_ACK;
	  I2C_InitStruct.OwnAddrSize 		 = LL_I2C_OWNADDRESS1_7BIT;


	  LL_I2C_Init(i2cBus->i2cPeripheral, &I2C_InitStruct);
	  LL_I2C_EnableAutoEndMode(i2cBus->i2cPeripheral);
	  LL_I2C_SetOwnAddress2(i2cBus->i2cPeripheral, 0, LL_I2C_OWNADDRESS2_NOMASK);
	  LL_I2C_DisableOwnAddress2(i2cBus->i2cPeripheral);
	  LL_I2C_DisableGeneralCall(i2cBus->i2cPeripheral);
	  LL_I2C_EnableClockStretching(i2cBus->i2cPeripheral);

	  /* Enable I2C transfer complete/error interrupts */
	  LL_I2C_EnableIT_TX(i2cBus->i2cPeripheral);
	  LL_I2C_EnableIT_RX(i2cBus->i2cPeripheral);
	  LL_I2C_EnableIT_TC(i2cBus->i2cPeripheral);
//	  LL_I2C_EnableIT_NACK(I2C1);
//	  LL_I2C_EnableIT_ERR(I2C1);
	  LL_I2C_EnableIT_STOP(i2cBus->i2cPeripheral);


	  // I2C Master successfully configured, return true.
	  //return ErrStatus;
}


/**
 * @brief  queue transaction is function that will be called from upper layer. This will add the transaction to the I2C condition queue
 * with correct parameters.
 * @param I2C peripheral, device addresss, register, buffer address, amount of bytes, type of transaction (READ/WRITE)
 * @retval True if function was successfull, else false.
 */


bool queue_i2c_transaction(I2C_TypeDef *I2Cx, uint8_t addr, uint8_t reg, uint8_t *buffer, uint8_t num_bytes, I2C_Read_Write_e type,
		void *start_callback, void *trans_callback, void *stop_callback, I2C_Last_e action)
{

	I2C_transaction_s temp_trans;
	uint8_t i2c_index = 0;

	temp_trans.i2cPeripheral = I2Cx;
	i2c_index = RETURN_I2C_ENUM(temp_trans.i2cPeripheral);

	if(List_getSize((List_t *)&i2c_transactions[i2c_index]) >= 20)
	{
		return false;
	}

	temp_trans.addr = addr;
	temp_trans.reg = reg;
	temp_trans.buffer = buffer;
	temp_trans.num_bytes= num_bytes;
	temp_trans.index = 0;
	temp_trans.trans_type = type;
	temp_trans.trans_last = false;
	temp_trans.trans_complete_action = action;
	temp_trans.I2C_T_State = I2C_TRANS_NONE;
	temp_trans.State = I2C_NONE;
	temp_trans.I2C_Timeout = 100;
	temp_trans.I2C_Timestamp = HAL_GetTick();
	temp_trans._start_callback = start_callback;
	temp_trans._trans_callback = trans_callback;
	temp_trans._stop_callback = stop_callback;

	/* add the transaction to the list for this perhiperhal */
	List_insertStart(&i2c_transactions[i2c_index], List_getSize((List_t *)&i2c_transactions[i2c_index]), &temp_trans);

	return true;
}


/**
  * @brief  Function called from I2C IRQ Handler when RXNE flag is set
  *         Function is in charge of reading byte received on I2C lines.
  * @param  None
  * @retval None
  */
bool Run_I2C_Transmission(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);
	I2C_transaction_s temp_trans;

	/* check if the list is empty */
	if(List_isEmpty((List_t *)&i2c_transactions[i2c_index]))
	{
		/* no I2C transactions are queued or the queue is busy and the are within the timeout */
		return false;
	}

	/* check if the peripheral is occupied */
	if((current_transaction[i2c_index].State == I2C_BUSY)
			&& (HAL_CheckTick(current_transaction[i2c_index].I2C_Timestamp, 5, false)))
	{
		/* the I2C peripheral is still busy */
		return false;
	}


	/* pop the last transaction from the list */
	List_getLast((List_t *)&i2c_transactions[i2c_index], &temp_trans);

	/* check if this is the last transaction */
	if(List_getSize((List_t *)&i2c_transactions[i2c_index]) == 1)
	{
		temp_trans.trans_last = true;
	}

	__disable_irq();
	/* copy over the transactions variable taken from the list into the transactions buffer */
	memcpy(&current_transaction[i2c_index], &temp_trans, sizeof(I2C_transaction_s));
	__enable_irq();

	/* remove the transaction from the list */
	List_deleteLast((List_t *)&i2c_transactions[i2c_index]);




	/* Start the I2C transfer */
	if (current_transaction[i2c_index].trans_type == I2C_TYPE_WRITE)
	{
		Start_I2C_Write_Transfer(current_transaction[i2c_index].i2cPeripheral);
	}
	else if (current_transaction[i2c_index].trans_type == I2C_TYPE_READ)
	{
		Start_I2C_Read_Transfer(current_transaction[i2c_index].i2cPeripheral);
	}

	/* set the transaction state indication to busy */
	current_transaction[i2c_index].State = I2C_BUSY;

	return true;
}


/**
 * @brief  return flag to indicated if I2C is busy
 * @param  None
 * @retval None
 */
bool i2c_busy(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	if (((current_transaction[i2c_index].I2C_T_State == I2C_TRANS_NONE)
		&&	HAL_CheckTick(current_transaction[i2c_index].I2C_Timestamp, 5, true))
			|| (HAL_CheckTick(current_transaction[i2c_index].I2C_Timestamp, current_transaction[i2c_index].I2C_Timeout, true)))
	{
		return true;
	}

	return false;
}

/**
  * @brief  This Function kicks off an I2C reading
  * @param  None
  * @retval None
  */
void Start_I2C_Read_Transfer(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);
	/* Configure slave address, nbytes, reload, end mode and start or stop generation */
	LL_I2C_HandleTransfer(I2Cx, (current_transaction[i2c_index].addr)<<1, LL_I2C_ADDRSLAVE_7BIT, 1, LL_I2C_MODE_SOFTEND, LL_I2C_GENERATE_START_WRITE);

	/* set state to write - write address state */
	current_transaction[i2c_index].I2C_T_State = I2C_TRANS_ADDRESS;
	/* reset the index */
	current_transaction[i2c_index].index = 0;

	if(current_transaction[i2c_index]._start_callback != NULL)
	{
		current_transaction[i2c_index]._start_callback();
	}

}

/**
  * @brief  This Function kicks off an I2C write
  * @param  None
  * @retval None
  */
void Start_I2C_Write_Transfer(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	/* copy over the buffer shifted by one */
	memcpy(&write_buffer[1], current_transaction[i2c_index].buffer, (current_transaction[i2c_index].num_bytes));

	/* set the desired register */
	write_buffer[0] = current_transaction[i2c_index].reg;

	/* write action */
	LL_I2C_HandleTransfer(I2Cx, (current_transaction[i2c_index].addr)<<1, LL_I2C_ADDRSLAVE_7BIT,
				current_transaction[i2c_index].num_bytes+1, LL_I2C_MODE_SOFTEND, LL_I2C_GENERATE_RESTART_7BIT_WRITE);

	/* set state to write - write address state */
	current_transaction[i2c_index].I2C_T_State = I2C_TRANS_WRITE_BYTE;
	/* reset the index */
	current_transaction[i2c_index].index = 0;

	if (current_transaction[i2c_index]._start_callback != NULL)
	{
		current_transaction[i2c_index]._start_callback();
	}

}

/**
  * @brief  Function called from I2C IRQ Handler when TXIS Flag is set
  * @param  None
  * @retval None
  */
void Transmission_Register_Callback(I2C_TypeDef *I2Cx)
{

	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	if(current_transaction[i2c_index].trans_type == I2C_TYPE_WRITE)
	{
		/* write action */
		current_transaction[i2c_index].I2C_T_State = I2C_TRANS_WRITE_BYTE;


	}
	else if(current_transaction[i2c_index].I2C_T_State == I2C_TRANS_ADDRESS)
	{
		/* Send Register address */
		LL_I2C_TransmitData8(I2Cx, (uint8_t)current_transaction[i2c_index].reg);

		current_transaction[i2c_index].I2C_T_State = I2C_TRANS_REGISTER;
	}

}

/**
  * @brief  Function called from I2C IRQ Handler when TXIS Flag is set
  * @param  None
  * @retval None
  */
void Transmission_Start_Callback(I2C_TypeDef *I2Cx)
{

	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);


	if(current_transaction[i2c_index].trans_type == I2C_TYPE_WRITE)
	{
		/* write action */
		current_transaction[i2c_index].I2C_T_State = I2C_TRANS_WRITE_BYTE;

		/* call the transmission callback() */
		Transmission_Callback(I2Cx);
	}

	else if(current_transaction[i2c_index].trans_type ==  I2C_TYPE_READ)
	{
		/* read action */
		/* write action */
		if(current_transaction[i2c_index].trans_last)
		{
			/* Configure slave address, nbytes, reload, end mode and start or stop generation */
			LL_I2C_HandleTransfer(I2Cx, (current_transaction[i2c_index].addr)<<1, LL_I2C_ADDRSLAVE_7BIT,
					current_transaction[i2c_index].num_bytes, LL_I2C_MODE_SOFTEND, LL_I2C_GENERATE_START_READ);
		}
		else
		{
			/* Configure slave address, nbytes, reload, end mode and start or stop generation */
			LL_I2C_HandleTransfer(I2Cx, (current_transaction[i2c_index].addr)<<1, LL_I2C_ADDRSLAVE_7BIT,
					current_transaction[i2c_index].num_bytes, LL_I2C_MODE_SOFTEND, LL_I2C_GENERATE_RESTART_7BIT_READ);
		}

		/* set the T_State to Byte if it is not there yet */
		if (current_transaction[i2c_index].I2C_T_State == I2C_TRANS_REGISTER)
		{
			current_transaction[i2c_index].I2C_T_State = I2C_TRANS_READ_BYTE;
		}
	}

}

/**
  * @brief  Function called from I2C IRQ Handler when RXNE flag is set
  *         Function is in charge of reading byte received on I2C lines.
  * @param  None
  * @retval None
  */
void Reception_Callback(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);
	// Populate the i2c buffer
	*(current_transaction[i2c_index].buffer + (current_transaction[i2c_index].index++)) = LL_I2C_ReceiveData8(I2Cx);

	/* check if the transaction is complete */
	if(current_transaction[i2c_index].num_bytes == current_transaction[i2c_index].index)
	{

		switch(current_transaction[i2c_index].trans_complete_action)
		{
			case I2C_LAST_FORCE_LAST:

				Stop_Transaction(I2Cx);

			break;

			case I2C_LAST_NO_FORCE:

				if(current_transaction[i2c_index].trans_last)
				{
					Stop_Transaction(I2Cx);
				}

			break;

			case I2C_LAST_WAIT:

				if(current_transaction[i2c_index]._trans_callback != NULL)
				{
					current_transaction[i2c_index]._trans_callback();
				}

			break;

		}
		current_transaction[i2c_index].State = I2C_COMPLETE;
	}
}

/**
  * @brief  Function called from I2C IRQ Handler when RXNE flag is set
  *         Function is in charge of reading byte received on I2C lines.
  * @param  None
  * @retval None
  */
void Transmission_Callback(I2C_TypeDef *I2Cx)
{

	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	/* Send Byte Data */
	LL_I2C_TransmitData8(I2Cx, write_buffer[current_transaction[i2c_index].index++]);

	if(current_transaction[i2c_index].trans_last)
	{
		/* check if the transaction is complete */
		if(current_transaction[i2c_index].index == (current_transaction[i2c_index].num_bytes+2))
		{
			switch(current_transaction[i2c_index].trans_complete_action)
			{
				case I2C_LAST_FORCE_LAST:

					Stop_Transaction(I2Cx);

				break;

				case I2C_LAST_NO_FORCE:

						Stop_Transaction(I2Cx);

				break;

				case I2C_LAST_WAIT:

					if(current_transaction[i2c_index]._trans_callback != NULL)
					{
						current_transaction[i2c_index]._trans_callback();
					}

				break;
			}

			current_transaction[i2c_index].State = I2C_COMPLETE;
		}
	}
	else
	{		/* check if the transaction is complete */
		if(current_transaction[i2c_index].index == (current_transaction[i2c_index].num_bytes+1))
		{
			switch(current_transaction[i2c_index].trans_complete_action)
			{
				case I2C_LAST_FORCE_LAST:

					Stop_Transaction(I2Cx);

				break;

				case I2C_LAST_NO_FORCE:

					if(current_transaction[i2c_index].trans_last)
					{

						Stop_Transaction(I2Cx);
					}

				break;

				case I2C_LAST_WAIT:

					if(current_transaction[i2c_index]._trans_callback != NULL)
					{
						current_transaction[i2c_index]._trans_callback();
					}

				break;
			}

			current_transaction[i2c_index].State = I2C_COMPLETE;
		}
	}
}


void Stop_Transaction(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	/* Configure slave address, nbytes, reload, end mode and start or stop generation */
	LL_I2C_HandleTransfer(I2Cx, (current_transaction[i2c_index].addr)<<1, LL_I2C_ADDRSLAVE_7BIT,
			0, LL_I2C_MODE_SOFTEND, LL_I2C_GENERATE_STOP);

	if(current_transaction[i2c_index]._stop_callback != NULL)
	{
		current_transaction[i2c_index]._stop_callback();
	}

}



/**
  * Brief   This function handles I2C1 interrupt request.
  * Param   None
  * Retval  None
  */
void I2C1_IRQHandler(void)
{


	switch (current_transaction[0].I2C_T_State)
	{

	case I2C_TRANS_ADDRESS:

		  /* Check TXIS flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TXIS(I2C1) || LL_I2C_IsActiveFlag_TXE(I2C1))
		  {
			/* Start Write Complete */
			LL_I2C_ClearFlag_TXE(I2C1);

			/* address transmission complete */

			/* Transmit the device Register */
			Transmission_Register_Callback(I2C1);
		  }

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TC(I2C1))
		  {
			/* Transmit the device Register */
			Transmission_Register_Callback(I2C1);
		  }

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TCR(I2C1))
		  {
			/* Transmit the device Register */
			Transmission_Register_Callback(I2C1);
		  }

		  /* Check NACK flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_NACK(I2C1))
		  {
			/* Clear the NACK flag */
			LL_I2C_ClearFlag_NACK(I2C1);

			current_transaction[0].I2C_T_State = I2C_TRANS_NONE; //todo transmission failed
			current_transaction[0].State = I2C_NONE;
		  }

		  /* Check STOP flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_STOP(I2C1))
		  {
			/* Clear the STOP flag */
			LL_I2C_ClearFlag_STOP(I2C1);

			current_transaction[0].I2C_T_State = I2C_TRANS_NONE; // todo transmission failed
			current_transaction[0].State = I2C_NONE;
		  }

		  //todo add I2C breakout option

	  break;

	case I2C_TRANS_REGISTER:

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TC(I2C1))
		  {
		    /* Start the I2C Reading loop */
			  Transmission_Start_Callback(I2C1);
		  }

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TCR(I2C1))
		  {
		    /* Start the I2C Reading loop */
			  Transmission_Start_Callback(I2C1);
		  }

	  break;

	case I2C_TRANS_WRITE_BYTE:

		  /* Check TXIS flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TXIS(I2C1) || LL_I2C_IsActiveFlag_TXE(I2C1))
		  {
			/* Start Write Complete */
			LL_I2C_ClearFlag_TXE(I2C1);

		    /* Call function Master Transmission Callback */
			  Transmission_Callback(I2C1);
		  }

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TC(I2C1))
		  {
		    /* Call function Master Transmission Callback */
			  Transmission_Callback(I2C1);

			  current_transaction[0].I2C_T_State = I2C_TRANS_DONE;
			  current_transaction[0].State = I2C_COMPLETE;
			  /* start the next transmission */
			  Run_I2C_Transmission(I2C1);
		  }

		  /* Check STOP flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_STOP(I2C1))
		  {
			  /* End of Transfer */
			    LL_I2C_ClearFlag_STOP(I2C1);

		      /*Clear the I2C flag */
			  current_transaction[0].I2C_T_State = I2C_TRANS_DONE;
			  current_transaction[0].State = I2C_COMPLETE;
			  current_transaction[0].I2C_Timestamp = HAL_GetTick();

		  }

	  break;

	case I2C_TRANS_READ_BYTE:

		  /* Check RXNE flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_RXNE(I2C1))
		  {
		    /* Call function Master Reception Callback */
			  Reception_Callback(I2C1);
		  }

		  /* Check TC flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_TC(I2C1))
		  {
		    /* receive is done */
			  current_transaction[0].State = I2C_COMPLETE;
			  current_transaction[0].I2C_T_State = I2C_TRANS_DONE;

			  /* start the next transmission */
			  Run_I2C_Transmission(I2C1);
		  }

		  /* Check STOP flag value in ISR register */
		  if(LL_I2C_IsActiveFlag_STOP(I2C1))
		  {
		    /* End of Transfer */
		    LL_I2C_ClearFlag_STOP(I2C1);
			/*Clear the I2C flag */
		    current_transaction[0].I2C_T_State = I2C_TRANS_DONE;
			current_transaction[0].State = I2C_COMPLETE;
			current_transaction[0].I2C_Timestamp = HAL_GetTick();

		  }

	  break;

	default:
		 	  /* Check RXNE flag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_RXNE(I2C1))
		 	  {

		 	  }

		 	  /* Check STOP flag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_STOP(I2C1))
		 	  {
		 		   /*Clear the I2C flag */
		 	    LL_I2C_ClearFlag_STOP(I2C1);
		 	  }

		 	  /* Check TC flag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_TC(I2C1))
		 	  {

		 	  }

		 	  /* Check TXEflag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_TXE(I2C1))
		 	  {
		 		/* Start Write Complete */
		 		LL_I2C_ClearFlag_TXE(I2C1);

		 	  }

		 	  /* Check TXIS flag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_TXIS(I2C1))
		 	  {
		 		/* Start Write Complete */
		 		LL_I2C_ClearFlag_TXE(I2C1);

		 	  }

		 	  /* Check NACK flag value in ISR register */
		 	  if(LL_I2C_IsActiveFlag_NACK(I2C1))
		 	  {
		 		/* Clear the NACK flag */
		 		LL_I2C_ClearFlag_NACK(I2C1);
		 	  }

		 	  LL_I2C_Disable(I2C1);

		 	  asm("nop");
		 	  asm("nop");
		 	  asm("nop");

		 	  LL_I2C_Enable(I2C1);


			  current_transaction[0].State = I2C_NONE;
	  break;
	}


	/* If the I2C Handler ends up in the none state, all flags are cleared */
	// else
	// {
	// 	  /* Check RXNE flag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_RXNE(I2C1))
	// 	  {

	// 	  }

	// 	  /* Check STOP flag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_STOP(I2C1))
	// 	  {
	// 		   /*Clear the I2C flag */
	// 	    LL_I2C_ClearFlag_STOP(I2C1);
	// 	  }

	// 	  /* Check TC flag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_TC(I2C1))
	// 	  {

	// 	  }

	// 	  /* Check TXEflag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_TXE(I2C1))
	// 	  {
	// 		/* Start Write Complete */
	// 		LL_I2C_ClearFlag_TXE(I2C1);

	// 	  }

	// 	  /* Check TXIS flag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_TXIS(I2C1))
	// 	  {
	// 		/* Start Write Complete */
	// 		LL_I2C_ClearFlag_TXE(I2C1);

	// 	  }

	// 	  /* Check NACK flag value in ISR register */
	// 	  if(LL_I2C_IsActiveFlag_NACK(I2C1))
	// 	  {
	// 		/* Clear the NACK flag */
	// 		LL_I2C_ClearFlag_NACK(I2C1);
	// 	  }

	// 	  LL_I2C_Disable(I2C1);

	// 	  asm("nop");
	// 	  asm("nop");
	// 	  asm("nop");

	// 	  LL_I2C_Enable(I2C1);


	// }

}

/**
  * Brief   This function handles I2C2 interrupt request.
  * Param   None
  * Retval  None
  */
//void I2C2_IRQHandler(void)
//{
//
//
//	switch (current_transaction[1].I2C_T_State)
//	{
//
//	case I2C_TRANS_ADDRESS:
//
//		  /* Check TXIS flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TXIS(I2C2))
//		  {
//			/* Start Write Complete */
//			LL_I2C_ClearFlag_TXE(I2C2);
//
//			/* Transmit the device Register */
//			Transmission_Register_Callback(I2C2);
//		  }
//
//		  /* Check TC flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TC(I2C2))
//		  {
//			/* Transmit the device Register */
//			Transmission_Register_Callback(I2C2);
//		  }
//
//		  /* Check TC flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TCR(I2C2))
//		  {
//			/* Transmit the device Register */
//			Transmission_Register_Callback(I2C2);
//		  }
//
//		  /* Check NACK flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_NACK(I2C2))
//		  {
//			/* Clear the NACK flag */
//			LL_I2C_ClearFlag_NACK(I2C2);
//
//			current_transaction[1].I2C_T_State = I2C_TRANS_NONE;
//		  }
//
//		  /* Check STOP flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_STOP(I2C2))
//		  {
//			/* Clear the STOP flag */
//			LL_I2C_ClearFlag_STOP(I2C2);
//
//			current_transaction[1].I2C_T_State = I2C_TRANS_NONE;
//		  }
//
//		  //todo add I2C breakout option
//
//	  break;
//
//	case I2C_TRANS_REGISTER:
//
//		  /* Check TC flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TC(I2C2))
//		  {
//		    /* Start the I2C Reading loop */
//			  Transmission_Start_Callback(I2C2);
//		  }
//
//		  /* Check TC flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TCR(I2C2))
//		  {
//		    /* Start the I2C Reading loop */
//			  Transmission_Start_Callback(I2C2);
//		  }
//
//	  break;
//
//	case I2C_TRANS_WRITE_BYTE:
//
//		  /* Check TXIS flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TXIS(I2C2))
//		  {
//			/* Start Write Complete */
//			LL_I2C_ClearFlag_TXE(I2C2);
//
//		    /* Call function Master Transmission Callback */
//			  Transmission_Callback(I2C2);
//		  }
//
//		  /* Check TC flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_TC(I2C2))
//		  {
//		    /* Call function Master Transmission Callback */
//			  Transmission_Callback(I2C2);
//		  }
//
//		  /* Check STOP flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_STOP(I2C2))
//		  {
//
//			  /* Callback for End of Transmission */
//			  Transmission_End_Callback(I2C2);
//
//		  }
//
//	  break;
//
//	case I2C_TRANS_READ_BYTE:
//
//		  /* Check RXNE flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_RXNE(I2C2))
//		  {
//		    /* Call function Master Reception Callback */
//			  Reception_Callback(I2C2);
//		  }
//
//		  /* Check STOP flag value in ISR register */
//		  if(LL_I2C_IsActiveFlag_STOP(I2C2))
//		  {
//		    /* End of Transfer */
//		    LL_I2C_ClearFlag_STOP(I2C2);
//
//		    /* Call function Master Reception End Callback */
//		    Reception_End_Callback(I2C2);
//
//		  }
//
//	  break;
//
//	default:
//	  break;
//	}
//
//
//	/* If the I2C Handler ends up in the none state, all flags are cleared */
//	// else
//	// {
//	// 	  /* Check RXNE flag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_RXNE(I2C2))
//	// 	  {
//
//	// 	  }
//
//	// 	  /* Check STOP flag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_STOP(I2C2))
//	// 	  {
//	// 		   /*Clear the I2C flag */
//	// 	    LL_I2C_ClearFlag_STOP(I2C2);
//	// 	  }
//
//	// 	  /* Check TC flag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_TC(I2C2))
//	// 	  {
//
//	// 	  }
//
//	// 	  /* Check TXEflag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_TXE(I2C2))
//	// 	  {
//	// 		/* Start Write Complete */
//	// 		LL_I2C_ClearFlag_TXE(I2C2);
//
//	// 	  }
//
//	// 	  /* Check TXIS flag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_TXIS(I2C2))
//	// 	  {
//	// 		/* Start Write Complete */
//	// 		LL_I2C_ClearFlag_TXE(I2C2);
//
//	// 	  }
//
//	// 	  /* Check NACK flag value in ISR register */
//	// 	  if(LL_I2C_IsActiveFlag_NACK(I2C2))
//	// 	  {
//	// 		/* Clear the NACK flag */
//	// 		LL_I2C_ClearFlag_NACK(I2C2);
//	// 	  }
//
//	// 	  LL_I2C_Disable(I2C2);
//
//	// 	  asm("nop");
//	// 	  asm("nop");
//	// 	  asm("nop");
//
//	// 	  LL_I2C_Enable(I2C2);
//
//
//	// }
//
//}
