/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    i2c.h							 					  	  	  *
 * @brief 	    Header for i2c driver								   		  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    28/03/2017                                                    *
 ******************************************************************************/

#ifndef __I2C_H__
#define __I2C_H__

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "list.h"
//#include "AZQ_util.h"
#include <stdbool.h>
/* Exported types ------------------------------------------------------------*/

/**
 * @brief  I2C State Definition
 */
typedef enum
{
	I2C_NONE 						= 	0x00U,
	I2C_BUSY		  				= 	0x01U,
	I2C_COMPLETE			  		= 	0x02U
} I2C_State_e;

/**
 * @brief  I2C Transaction State Definition
 */
typedef enum
{
	I2C_TRANS_NONE 						= 	0x00U,
	I2C_TRANS_START  					= 	0x01U,
	// write address
	I2C_TRANS_ADDRESS 					= 	0x02U,
	I2C_TRANS_ADDRESS_16				= 	0x03U,
	// write register
	I2C_TRANS_REGISTER		  			= 	0x04U,
	I2C_TRANS_REGISTER_16_1	  			= 	0x05U,
	I2C_TRANS_REGISTER_16_2	  			= 	0x06U,
	// read / write byte
	I2C_TRANS_READ_BYTE			  		= 	0x07U,
	I2C_TRANS_READ_BYTE_16		  		= 	0x08U,
	I2C_TRANS_WRITE_BYTE			  	= 	0x09U,
	I2C_TRANS_DONE  					= 	0x0AU
} I2C_Transaction_State_e;

/**
 * @brief  I2C Data Definition
 */
typedef enum
{
	I2C_DATA_NONE     				= 	0x00U,
	I2C_DATA_READING  				= 	0x01U,
	I2C_DATA_DONE		 			= 	0x02U,
	I2C_DATA_TIMEOUT				=   0x03U
} I2C_Data_e;

typedef enum
{
	I2C_MODE_STANDARD						=  0x00U,
	I2C_MODE_FAST							=  0x01U,
	I2C_MODE_FASTPLUS						=  0x02U
} I2C_Mode_e;

/**
 * @brief I2C HardwareBus Definition
 */


#pragma pack(1)
typedef struct {
	I2C_TypeDef 	*i2cPeripheral;
	GPIO_TypeDef	*sdaPort;
	uint32_t		sdaPin;
	uint32_t		sdaMode;
	uint32_t		sda_AF;
	GPIO_TypeDef	*sclPort;
	uint32_t		sclPin;
	uint32_t		sclMode;
	uint32_t		scl_AF;
	uint32_t		GPIOspeed;
	I2C_Mode_e		i2cMode;
	uint32_t		priority;
	uint8_t			i2cTimeout_ms;
} I2C_HardwareBus_t;
#pragma pack(4)


/**	@brief 	An enumerator specifying the various I2C error conditions
 *
 */
typedef enum {
	I2C_STATUS_SUCCESS 			= 0x00,
	I2C_STATUS_NACK				= 0x01,
	I2C_STATUS_FAIL 			= 0x02,
	I2C_STATUS_TIMEOUT 			= 0x04,
	I2C_STATUS_BUSY				= 0x08,
	I2C_STATUS_INVALID_PARAM 	= 0x10
} I2C_Err_Status_e;


/**
 * @brief I2C variable struct. Used to define variables for I2C peripherals used in comms.
 */

//#pragma pack(1)
//typedef struct {
//	uint8_t 			Tx_Idx;
//	uint8_t 			Rx_Idx;
//	uint8_t 			receiveBuffer[256];
//	uint8_t 			transmitBuffer[256];
//	uint8_t 			NumberOfBytesToReceive;	 // set this to change the amount of bits to receive
//	uint8_t 			NumberOfBytesToTransmit; // set this to change the amount of bits to transmit
//
//} I2C_Com_Variables_t;
//#pragma pack(4)



/**	@brief    An enumerator which allows selection of the End Condition for a
  * 					transaction.
  * @details  Allows the user to specify whether the transaction must end in a
  *           Stop command or a Restart command.
  */
typedef enum {
	I2C_END_RESTART			= 0x00,
	I2C_END_STOP		    = 0x01
} I2C_EndCondition_e;


/**	@brief    An enumerator which allows selection of the I2C transaction type when queueing transactions.
  * @details
  */
typedef enum {
	I2C_TYPE_NONE			= 0x00,
	I2C_TYPE_WRITE			= 0x01,
	I2C_TYPE_READ		    = 0x02
} I2C_Read_Write_e;

typedef enum {
	I2C_LAST_NO_FORCE			= (uint8_t) 0x00,
	I2C_LAST_WAIT				= 0x01,
	I2C_LAST_FORCE_LAST		    = 0x02
} I2C_Last_e;

typedef enum{
  I2C_TRANS_REPEAT,
  I2C_TRANS_DELETE
} I2C_transaction_key_e;

typedef enum{
  I2C_TRANS_FIRST,
  I2C_TRANS_LAST
} I2C_trans_position_e;



#pragma pack(1)
typedef struct {
	I2C_TypeDef 				*i2cPeripheral;
	uint8_t 					addr;
	uint8_t						reg;
//	uint16_t					reg16;
	uint8_t	 					*buffer;
	uint8_t 					num_bytes;		// bytes * 2 for words
	uint8_t						index;
	uint32_t					I2C_Timeout;
	uint32_t					I2C_Timestamp;
	bool						trans_last;
	I2C_Last_e					trans_complete_action;
	I2C_Read_Write_e 			trans_type;
	I2C_Transaction_State_e 	I2C_T_State;
	I2C_State_e					State;
	I2C_transaction_key_e 		trans_key;
	I2C_trans_position_e		trans_position;
	void						(*_start_callback)(void);
	void						(*_trans_callback)(void);
	void						(*_stop_callback)(void);
} I2C_transaction_s;
#pragma pack(4)

/* Exported constants --------------------------------------------------------*/

/* Timing register value is computed with the STM32CubeMX Tool,
  * Fast Mode @400kHz with I2CCLK = 56 MHz,
  * rise time = 100ns, fall time = 10ns
  * Timing Value = (uint32_t)0x00F02B86
  */

/* Pin Defines */

//#define I2Cx_SDA_AF                 LL_GPIO_AF_6
//#define I2Cx_SCL_AF                 LL_GPIO_AF_6



#define I2C_CLOCK_SPEED				LL_RCC_GetI2CClockFreq(LL_RCC_I2C1_CLKSOURCE)

// Timing register clock classification limits

#define I2C_CLK_LOW			 10e6
#define I2C_CLK_MID			 36e6



// Timing register values for Standard mode I2C (100kHz), I2C clock smaller than I2C_CLK_LOW

#define SM_LOWC_PRESC		 0x1
#define SM_LOWC_SCLDEL		 0x4
#define SM_LOWC_SDADEL		 0x2
#define SM_LOWC_SCLH		 0xF
#define SM_LOWC_SCLL		 0x13

// Timing register values for Standard mode I2C (100kHz), I2C clock between I2C_CLK_LOW and I2C_CLK_MID

#define SM_MIDC_PRESC		 0x3
#define SM_MIDC_SCLDEL		 0x4
#define SM_MIDC_SDADEL		 0x2
#define SM_MIDC_SCLH		 0xF
#define SM_MIDC_SCLL		 0x13

// Timing register values for Standard mode I2C (100kHz), I2C clock greater than I2C_CLK_MID

#define SM_HIGHC_PRESC		 0xB
#define SM_HIGHC_SCLDEL		 0x4
#define SM_HIGHC_SDADEL		 0x2
#define SM_HIGHC_SCLH		 0xF
#define SM_HIGHC_SCLL		 0x13


// Timing register values for Fast mode I2C (400kHz), I2C clock smaller than I2C_CLK_LOW

#define FM_LOWC_PRESC		 0x0
#define FM_LOWC_SCLDEL		 0x3
#define FM_LOWC_SDADEL		 0x1
#define FM_LOWC_SCLH		 0x3
#define FM_LOWC_SCLL		 0x9

// Timing register values for Fast mode I2C (400kHz), I2C clock between I2C_CLK_LOW and I2C_CLK_MID

#define FM_MIDC_PRESC		 0x1
#define FM_MIDC_SCLDEL		 0x3
#define FM_MIDC_SDADEL		 0x2
#define FM_MIDC_SCLH		 0x3
#define FM_MIDC_SCLL		 0x9

// Timing register values for Fast mode I2C (400kHz), I2C clock greater than I2C_CLK_MID

#define FM_HIGHC_PRESC		 0x5
#define FM_HIGHC_SCLDEL		 0x3
#define FM_HIGHC_SDADEL		 0x3
#define FM_HIGHC_SCLH		 0x3
#define FM_HIGHC_SCLL		 0x9


// Timing register values for Fast plus mode I2C (1000kHz), I2C clock smaller than I2C_CLK_LOW

#define FMP_LOWC_PRESC		 0x0
#define FMP_LOWC_SCLDEL		 0x1
#define FMP_LOWC_SDADEL		 0x0
#define FMP_LOWC_SCLH		 0x3
#define FMP_LOWC_SCLL		 0x6

// Timing register values for Fast plus mode I2C (1000kHz), I2C clock between I2C_CLK_LOW and I2C_CLK_MID

#define FMP_MIDC_PRESC		 0x0
#define FMP_MIDC_SCLDEL		 0x2
#define FMP_MIDC_SDADEL		 0x0
#define FMP_MIDC_SCLH		 0x2
#define FMP_MIDC_SCLL		 0x4

// Timing register values for Fast plus mode I2C (1000kHz), I2C clock greater than I2C_CLK_MID

#define FMP_HIGHC_PRESC		 0x5
#define FMP_HIGHC_SCLDEL	 0x1
#define FMP_HIGHC_SDADEL	 0x0
#define FMP_HIGHC_SCLH		 0x1
#define FMP_HIGHC_SCLL		 0x3




#define 					IS_I2C1(X) (((X)==I2C1_BASE))
#define 					IS_I2C2(X) (((X)==I2C2_BASE))
#define 					IS_NOT_I2C(X) (((X)!=I2C1_BASE) && ((X)!=I2C2_BASE))
// Function to generate index for I2C_Com_Variable array according to I2C peripheral selected. Cast Peripheral to uint32 in order to compare
#define 					RETURN_I2C_ENUM(X) (0*(IS_I2C1((uint32_t)X)) + 1*(IS_I2C2((uint32_t)X) + 2*IS_NOT_I2C((uint32_t)X)))
// Function to determine whether Peripheral selection for I2C is valid
#define 					IS_I2C_VALID(X) ((((uint32_t)X)==I2C1_BASE) || (((uint32_t)X)==I2C2_BASE))

// I2C timing macro
#define						PRESC(X)  ((X == I2C_MODE_STANDARD)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? SM_LOWC_PRESC : (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? SM_MIDC_PRESC : (I2C_CLOCK_SPEED > I2C_CLK_MID)?SM_HIGHC_PRESC:0x00):(X == I2C_MODE_FAST)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FM_LOWC_PRESC : \
									  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FM_MIDC_PRESC : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FM_HIGHC_PRESC:0x00):(X == I2C_MODE_FASTPLUS)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FMP_LOWC_PRESC : \
									  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FMP_MIDC_PRESC : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FMP_HIGHC_PRESC:0x00):0x00)

#define						SCLDEL(X) ((X == I2C_MODE_STANDARD)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? SM_LOWC_SCLDEL : (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? SM_MIDC_SCLDEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?SM_HIGHC_SCLDEL:0x00):(X == I2C_MODE_FAST)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FM_LOWC_SCLDEL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FM_MIDC_SCLDEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FM_HIGHC_SCLDEL:0x00):(X == I2C_MODE_FASTPLUS)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FMP_LOWC_SCLDEL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FMP_MIDC_SCLDEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FMP_HIGHC_SCLDEL:0x00):0x00)

#define						SDADEL(X) ((X == I2C_MODE_STANDARD)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? SM_LOWC_SDADEL : (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? SM_MIDC_SDADEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?SM_HIGHC_SDADEL:0x00):(X == I2C_MODE_FAST)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FM_LOWC_SDADEL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FM_MIDC_SDADEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FM_HIGHC_SDADEL:0x00):(X == I2C_MODE_FASTPLUS)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FMP_LOWC_SDADEL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FMP_MIDC_SDADEL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FMP_HIGHC_SDADEL:0x00):0x00)

#define						SCLH(X)	  ((X == I2C_MODE_STANDARD)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? SM_LOWC_SCLH : (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? SM_MIDC_SCLH : (I2C_CLOCK_SPEED > I2C_CLK_MID)?SM_HIGHC_SCLH:0x00):(X == I2C_MODE_FAST)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FM_LOWC_SCLH : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FM_MIDC_SCLH : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FM_HIGHC_SCLH:0x00):(X == I2C_MODE_FASTPLUS)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FMP_LOWC_SCLH : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FMP_MIDC_SCLH : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FMP_HIGHC_SCLH:0x00):0x0)

#define						SCLL(X)	  ((X == I2C_MODE_STANDARD)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? SM_LOWC_SCLL : (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? SM_MIDC_SCLL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?SM_HIGHC_SCLL:0x00):(X == I2C_MODE_FAST)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FM_LOWC_SCLL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FM_MIDC_SCLL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FM_HIGHC_SCLL:0x00):(X == I2C_MODE_FASTPLUS)?((I2C_CLOCK_SPEED < I2C_CLK_LOW) ? FMP_LOWC_SCLL : \
		  	  	  	  	  	  	  	  (I2C_CLOCK_SPEED > I2C_CLK_LOW && I2C_CLOCK_SPEED < I2C_CLK_MID)? FMP_MIDC_SCLL : (I2C_CLOCK_SPEED > I2C_CLK_MID)?FMP_HIGHC_SCLL:0x00):0x00)








/* Exported functions ------------------------------------------------------- */
void init_i2c(I2C_HardwareBus_t *i2cBus);
bool Run_I2C_Transmission(I2C_TypeDef *I2Cx);
void Stop_Transaction(I2C_TypeDef *I2Cx);
bool queue_i2c_transaction(I2C_TypeDef *I2Cx, uint8_t addr, uint8_t reg, uint8_t *buffer, uint8_t num_bytes, I2C_Read_Write_e type,
		void *start_callback, void *trans_callback, void *stop_callback, I2C_Last_e forcelast);
bool CheckStop(void);


#endif /* __I2C_H__ */
