/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    iqs269.c							 				  	 	  *
 * @brief 	    iqs269 driver file								  		  	  *
 * @author 		CA van der Merwe, FP Stegmann - Azoteq PTY Ltd                *
 * @version 	V1.0.0														  *
 * @note		Any functions or variables which are meant to be used		  *
 * 				outside of this file are preceeded with 'iqs269_' in order	  *
 * 				to identify them when used in other files.                    *                                	  
 * @date 	    21/03/2020                                                    *
 ******************************************************************************/


/* Includes ------------------------------------------------------------------*/

#include  "iqs269.h"
#include  <stdlib.h>


/* Pointer declared to use for iqs269_s received from application layer */
iqs269_s *_iqsPtr;


/** 
 * @name	init_iqs269_MM
 * @brief	This function initiates the iqs269 Memory Map which is stored on board the STM32.
 * 			This function assigns the memory map pointer to the correct instance. The actual
 * 			setting values must be assigned from the application layer, or else chip defaults
 * 			will be used.
 *
 *
 * @param	None
 *
 * @retval	\b None
 *
 *
 */
void init_iqs269_MM(void)
{
	_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_prod_info;
	_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.starting_register = IQS269_MM_PRODUCT_NUMBER;
	_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.length = sizeof(IQS269_PROD_INFO);


	_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_sys_ge_flags;
	_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.starting_register = IQS269_MM_SYSTEM_FLAGS;
	_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.length = sizeof(IQS269_SYS_GE_FLAGS);

	_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_gesture_ev;
	_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.starting_register = IQS269_MM_GESTURE_EVENTS;
	_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.length = sizeof(IQS269_GESTURE_EV);

	_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_chan_states;
	_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.starting_register = IQS269_MM_PROX_STATE;
	_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.length = sizeof(IQS269_CHAN_STATES);

	_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_filtered_counts;
	_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.starting_register = IQS269_MM_CNT_CH0_L;
	_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.length = sizeof(IQS269_FILTERED_COUNTS);

	_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_dta;
	_iqsPtr->iqs269_memsec.iqs269_dta_memsec.starting_register = IQS269_MM_CH0_DTA_L;
	_iqsPtr->iqs269_memsec.iqs269_dta_memsec.length = sizeof(IQS269_DELTA_COUNTS);

	_iqsPtr->iqs269_memsec.iqs269_ref_chan_delta_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ref_chan_delta;
	_iqsPtr->iqs269_memsec.iqs269_ref_chan_delta_memsec.starting_register = IQS269_MM_DCF_DTA_CH0_L;
	_iqsPtr->iqs269_memsec.iqs269_ref_chan_delta_memsec.length = sizeof(IQS269_REF_CHAN_DELTA);

	_iqsPtr->iqs269_memsec.iqs269_reserved_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_reserved;
	_iqsPtr->iqs269_memsec.iqs269_reserved_memsec.starting_register = IQS269_MM_MOV_CNTR_CH0;
	_iqsPtr->iqs269_memsec.iqs269_reserved_memsec.length = sizeof(IQS269_RESERVED);

	_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_slider_co;
	_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.starting_register = IQS269_MM_COORDINATE_SLDR0;
	_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.length = sizeof(IQS269_SLIDER_CO);

	_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_abs_cap;
	_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.starting_register = IQS269_MM_ABSCAP_BASE_L;
	_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.length = sizeof(IQS269_ABS_CAP);

	_iqsPtr->iqs269_memsec.iqs269_hall_call_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_hall_call;
	_iqsPtr->iqs269_memsec.iqs269_hall_call_memsec.starting_register = IQS269_MM_CALIB_VALUE0_L;
	_iqsPtr->iqs269_memsec.iqs269_hall_call_memsec.length = sizeof(IQS269_HALL_CALL);


    _iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_pow_sys_settings;
	_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register = IQS269_MM_PMU_SETTINGS;
	_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.length = sizeof(IQS269_POWER_SYS_SETTINGS);


	_iqsPtr->iqs269_memsec.iqs269_rep_timing_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_rep_timing;
	_iqsPtr->iqs269_memsec.iqs269_rep_timing_memsec.starting_register = IQS269_MM_REPORT_RATE_NP;
	_iqsPtr->iqs269_memsec.iqs269_rep_timing_memsec.length = sizeof(IQS269_REPORT_AND_TIMINGS);

	_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_global_settings;
	_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.starting_register = IQS269_MM_PXS_GLBL_SETTINGS_0;
	_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.length = sizeof(IQS269_GLOBAL_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch0_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.starting_register = IQS269_MM_CH0_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.length = sizeof(IQS269_CH0_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch1_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch1_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch1_settings_memsec.starting_register = IQS269_MM_CH1_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch1_settings_memsec.length = sizeof(IQS269_CH1_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch2_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch2_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch2_settings_memsec.starting_register = IQS269_MM_CH2_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch2_settings_memsec.length = sizeof(IQS269_CH2_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch3_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch3_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch3_settings_memsec.starting_register = IQS269_MM_CH3_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch3_settings_memsec.length = sizeof(IQS269_CH3_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch4_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch4_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch4_settings_memsec.starting_register = IQS269_MM_CH4_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch4_settings_memsec.length = sizeof(IQS269_CH4_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch5_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch5_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch5_settings_memsec.starting_register = IQS269_MM_CH5_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch5_settings_memsec.length = sizeof(IQS269_CH5_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch6_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch6_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch6_settings_memsec.starting_register = IQS269_MM_CH6_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch6_settings_memsec.length = sizeof(IQS269_CH6_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_ch7_settings_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_ch7_settings;
	_iqsPtr->iqs269_memsec.iqs269_ch7_settings_memsec.starting_register = IQS269_MM_CH7_SELECT_CRX;
	_iqsPtr->iqs269_memsec.iqs269_ch7_settings_memsec.length = sizeof(IQS269_CH7_SETTINGS);

	_iqsPtr->iqs269_memsec.iqs269_i2c_control_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_i2c_control;
	_iqsPtr->iqs269_memsec.iqs269_i2c_control_memsec.starting_register = IQS269_MM_I2C_CONTROL_SETTINGS;
	_iqsPtr->iqs269_memsec.iqs269_i2c_control_memsec.length = sizeof(IQS269_I2C_CONTROL);

	_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer = &_iqsPtr->IQSMemoryMap.iqs269_hall_ui_en;
	_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.starting_register = IQS269_MM_HALL_UI_ENABLE;
	_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.length = sizeof(IQS269_HALL_UI_ENABLE);



}



/** 
 * @name	init_iqs269	
 * @brief	Initialises the iqs269 device
 *
 * @param	iqs269_s : \b iqs269 - Pointer to iqs269 device which should be defined in upper application levels. 
 *
 * @retval	\b None
 *
 */
void init_iqs269(iqs269_s *iqs269)
{

	/* Assign passed argument to local pointer */
	_iqsPtr = iqs269;

	/* Initialise azq_device */

	List_init(&_iqsPtr->iqs269_queue, sizeof(I2C_transaction_s),20);

    azq_init(&_iqsPtr->azq_iqs269);

    /* Update initialise state */

	_iqsPtr->init_state = IQS269_INIT_INTERFACE;

}


/** 
 * @name	run_init_iqs269
 * @brief	Run initialization routine for iqs269
 *
 * @param	None
 *
 * @return	\b bool
 *
 */
bool run_init_iqs269(void)
{
	/* check whether device was assigned to pointer and initialization has started */
	if(_iqsPtr == NULL)
	{
		return false;
	}


	switch (_iqsPtr->init_state)
	{
		case IQS269_INIT_NONE:

			break;

		case IQS269_INIT_INTERFACE:

			/* init the memory map of the device */
			init_iqs269_MM();

			/* reset time stamp */
			_iqsPtr->update_timestamp = HAL_GetTick();

			/* Enter chip reset state */
			_iqsPtr->init_state = IQS269_INIT_CHIP_RESET;

			break;

		/* STEP 1 : reset IQS269 chip */
		case IQS269_INIT_CHIP_RESET:


			/* Set reset flag in iqsPtr */
			iqs269_setbit_SoftReset();


			/* Check whether comms window is open. Queue chip reset transaction and run transmission */
			if(_iqsPtr->openwindow)
			{

				queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register, 
						&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
						2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE);


				/* Run transmission of transaction queue. True return value indicates transmission has started */
				if(Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral))
				{
					/* Set parameter to indicate that job has started */
					iqs269_start_job();

					/* goto init read reset, this is will check whether the chip has successfully reset */
					_iqsPtr->init_state = IQS269_INIT_READ_RESET;

					/* reset the timestamp */
					_iqsPtr->update_timestamp = HAL_GetTick();

					/* reset the reset flag setting in iqsPtr*/
					iqs269_resetbit_SoftReset();

				}
			}
			break;

		/* STEP 2 :  Read first two bytes of Global Flags to confirm chip has successfully reset */
		case IQS269_INIT_READ_RESET:

			/* Check whether comms window is open */
			if(_iqsPtr->openwindow)
			{

				/* Check whether the manually selected RESET delay has passed (IQS269_WAIT_FOR_RESET_DELAY parameter) */
				if(HAL_CheckTick(_iqsPtr->update_timestamp, IQS269_WAIT_FOR_RESET_DELAY, true))
				{
					/* queue transaction to read global flags to confirm that chip has gone through reset event */
					queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
							_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.starting_register,
							&((IQS269_PROD_INFO *)(_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.section_pointer))->buffer[0],
							6, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE);

						/* Run transmission of transaction queue */
						if(Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral))
						{
							/* Set flag to indicate job queue has started */
							iqs269_start_job();
							/* goto init check reset State */
							_iqsPtr->init_state = IQS269_INIT_CHECK_RESET;
						}
				}

			}

			break;

		case IQS269_INIT_CHECK_RESET:

			/* Check whether last transaction of previous queue is complete */
			if(_iqsPtr->jobdone == true)
			{
				/* If flag shows that reset has happened, go to write settings state */
				if(iqs269_checkSoftReset())
				{
					_iqsPtr->init_state = IQS269_INIT_PMU_SETTINGS;

					/* Clear Show Reset flag in local _iqsPtr */
					iqs269_resetbit_ShowReset();

				}
				else
				{
					/* Reset failed, go back to Chip reset state to retry */
					_iqsPtr->init_state = IQS269_INIT_CHIP_RESET;
				}
			}

			break;

		/* STEP 3 : Once chip has reset, write all settings from H file to chip		 */
			case IQS269_INIT_PMU_SETTINGS:

				/* Check whether comms window is open */
					if(_iqsPtr->openwindow)
					{
						/* queue transaction to write all settings to chip */
						queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
								_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
								&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
								136, I2C_TYPE_WRITE, &iqs269_close_window, NULL,  &iqs269_end_job, 	I2C_LAST_NO_FORCE);

						/* Run transmission of transaction queue */
						if(Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral))
						{
							/* Set flag to indicate job queue has started */
							iqs269_start_job();

							/* if transaction has started, move to ACK_RESET state */
							_iqsPtr->init_state = IQS269_INIT_ACK_RESET;

						}

					}
				break;


		/* STEP 4 : acknowledge the reset on the IQS269 chip */
		case IQS269_INIT_ACK_RESET:


			/* Check whether comms window is open */
				if(_iqsPtr->openwindow)
				{

					/*Set ACK RESET bit in iqsPtr */
					iqs269_setbit_AckReset();

					/* Queue Transaction to send ACK_RESET */
					queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
							_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
							&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
							2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE);

					/* Run transmission of transaction queue */
						if(Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral))
						{
							/* Set flag to indicate job queue has started */
							iqs269_start_job();

							/* reset ACK_RESET bit in _iqsPtr */
							iqs269_resetbit_AckReset();

							/* Move to INIT_ATI state */
							_iqsPtr->init_state = IQS269_INIT_ATI;

						}
				}
			break;

		/* STEP 5 : ATI all channels after initial setup is complete */
		case IQS269_INIT_ATI:

			/* Check whether comms window is open and previous job is done */
			if(_iqsPtr->jobdone && _iqsPtr->openwindow)
			{
				/* Set the ATI bit in _iqsPtr */
				iqs269_setbit_ATI();

				/* Set reseed byte to indicate which channels need to be ATI'd */
				iqs269_setReseedByte(0xFF);

				if(_iqsPtr->jobdone)
				{


					queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
							IQS269_MM_RESEED_CHANNELS,
							  &((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_swipe_thres_slid,
							2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE);

					/* Queue ATI command transaction */
					queue_i2c_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
							_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
							((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer,
							2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE);

					/* Run transmission of transaction queue */
						if(Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral))
						{
							/* Set flag to indicate job queue has started */
							iqs269_start_job();


							/* Move to INIT_ATI state */
							_iqsPtr->init_state = IQS269_INIT_DONE;

						}
					}
			}

		break;


		/* STEP 5 : Once ack reset is done, move chip to idle state */
		case IQS269_INIT_DONE:

			if(_iqsPtr->jobdone)
			{
				/* Reset ATI bit in _iqsPtr */
				iqs269_resetbit_ATI();

				/* Assign reseed byte again once initial ATI is complete */
				iqs269_setReseedByte(0x00);

				/* Initialization is done, put chip into Idle Mode */
				_iqsPtr->state = IQS269_STATE_IDLE;

				/* Call function to queue transactions which will be executed on every comms window */
				iqs269_QueueValueUpdates();
			}

			break;

	}
	return true;
}




/**
 * @name	run_iqs269
 * @brief	Run the iqs269. This function is called from the main while loop and is the only function necessary to active the iqs269
 *
 * @param	None
 *
 *
 * @return	\b IQS269_update_e - indicates at what point in the state machine the program is.
 */
IQS269_update_e run_iqs269(void)
{
	/* Temporary I2C transcation variable */
	I2C_transaction_s temp_trans_run;

	/* Temporary variables to determine queue size and execution position */
	uint8_t q_size;
	uint8_t q_execute;

	/* check whether device was assigned to pointer */

	if (_iqsPtr == NULL)
	{
		return true;
	}

	/* Run I2C transaction queue */
	Run_I2C_Transmission(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral);

	/* IQS269 Run State Machine */
	switch(_iqsPtr->state)
		{
			/* The IQS269 IC has not yet been initialized */
			case IQS269_STATE_NONE:

				_iqsPtr->state = IQS269_STATE_INIT;

				return IQS269_UPDATE_NONE;
				break;

			/* Run the initialization of the IQS269 IC */
			case IQS269_STATE_INIT:

				run_init_iqs269();

				return IQS269_UPDATE_INIT;
				break;

			/*  Once initialized correctly, should be in Idle state */
			case IQS269_STATE_IDLE:

			/* The chip is removed from this state by the ready interrupt call back */

				break;

			/* Wait state can be used as a work around to wait for transactions to be completed */
			case IQS269_STATE_WAIT:

				if(_iqsPtr->jobdone)
				{
					_iqsPtr->state = IQS269_STATE_IDLE;
				}

			break;


			/* New data can be read from the IQS269 IC, this state is slected in the ready interrupt */
			case IQS269_STATE_DATA_READY:

				/* Check whether comms window is open */
				if(_iqsPtr->openwindow)
				{
					/* get list size before starting to manipulate list */
					q_size = List_getSize((List_t *)&_iqsPtr->iqs269_queue);
					q_execute = 0;
					/* queue every transaction in the iqs list to the i2c list for execution */
					for(_iqsPtr->queue_index = 0; _iqsPtr->queue_index < q_size; _iqsPtr->queue_index++)
					{
						/* Check whether iqs queue has a transaction to execute and retrieve transaction data */
						if(Run_IQS_Queue(&temp_trans_run))
						{
							/* queue transaction on i2c execution list */
							queue_i2c_transaction(temp_trans_run.i2cPeripheral, temp_trans_run.addr, temp_trans_run.reg, temp_trans_run.buffer,
									temp_trans_run.num_bytes, temp_trans_run.trans_type, temp_trans_run._start_callback, temp_trans_run._trans_callback,
									temp_trans_run._stop_callback, temp_trans_run.trans_complete_action);

							q_execute++;
							/* Check whether force last command was sent in transaction and break from loop if received */
							if(temp_trans_run.trans_complete_action == 	I2C_LAST_FORCE_LAST)
							{

								break;
							}
						}
					}

					/* Run through the list and delete all one-time transactions, keep repeat transactions */
					for(_iqsPtr->queue_index =  (q_execute-1); _iqsPtr->queue_index >= 0 ; _iqsPtr->queue_index--)
					{
						if(Run_IQS_Queue(&temp_trans_run))
						{
							if(temp_trans_run.trans_key == I2C_TRANS_DELETE)
							{
								List_deleteItem((List_t *)&_iqsPtr->iqs269_queue, _iqsPtr->queue_index);
							}
						}
					}

				}

				break;
		}
	return false;
}

/** 
 * @name	iqs269_QueueValueUpdates
 * @brief	This function is used to list the transactions which must be executed upon every available comms window.
 * 	   		These transactions are queued once the device is initialized. 
 * 			The transactions must be specified by the user, to avoid unnecessary communications. 
 * 			The function contains example transactions. 
 * @param	None
 *
 * @retval	\b bool - Indicates whether transactions were queued successfully. 
 */
bool iqs269_QueueValueUpdates(void)
{
	/*	Example queued transactions for user to emulate for application. */


	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_SYSTEM_FLAGS,
				&((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))->buffer[0],
				10, I2C_TYPE_READ, &iqs269_close_window, &iqs269_trans_callback, &iqs269_end_job, I2C_LAST_NO_FORCE, I2C_TRANS_REPEAT, I2C_TRANS_LAST);

	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_CH0_DTA_L,
					&((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch0_dta_ct_1,
					16, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE, I2C_TRANS_REPEAT, I2C_TRANS_LAST);

	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_COORDINATE_SLDR0,
				&((IQS269_SLIDER_CO *)(_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.section_pointer))->buffer[0],
				10, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_REPEAT, I2C_TRANS_LAST);





	return true;
}

/** 
 * @name	iqs269_deQueueValueUpdates
 * 
 * @brief	This function is used to empty the iqs269 queue. Typically this will be used to clear a comms window
 * 			of value updates to execute certain commands. It is important to use iqs269_QueueValueUpdates to enqueue the
 * 			value updates again after using this function.
 * 
 * @param	None
 *
 *
 * @retval \b bool - indicates that the queue has successfully been emptied.
 */
bool iqs269_deQueueValueUpdates(void)
{
	List_empty(&_iqsPtr->iqs269_queue);

	return true;
}

/** 
 * @name	Run_IQS_Queue
 * 
 * @brief	This function is used to access a specific item in the IQS queue using the queue index parameter. 
 * 
 * @param	I2C_transaction_s : \b *temp_trans	- addres of variable in which returned transaction data must be stored. 
 *
 *
 * @retval	\b bool	- indicates whether data was successfully moved to temporary transaction address which was provided. 
 */
bool Run_IQS_Queue(I2C_transaction_s *temp_trans)
{

	/* check if the list is empty */
	if(List_isEmpty((List_t *)&_iqsPtr->iqs269_queue))
	{
		/* no iqs transactions are queued */
		return false;
	}

	/* retrieve transaction data from queue using queue index parameter */
	List_getItem((List_t *)&_iqsPtr->iqs269_queue, _iqsPtr->queue_index, temp_trans);

	return true;
}

/**
 * @name	iqs_queue_transaction
 * @brief	This function adds a I2C transaction to the IQS device queue.
 *
 * @param	I2C_TypeDef : \b I2Cx - I2C Instance.
 * @param	uint8_t : \b addr - Specifies the slave address to be programmed.
 * @param	uint8_t : \b reg -  Specifies the register
 * @param	uint8_t : \b *buffer - Specifies the buffer which must be read from or to when completing transaction
 * @param	uint8_t : \b num_bytes - Specifies the amount of bytes that must be read/write
 * @param	I2C_Read_Write_e : \b type -  Specifies type of transaction from I2C_Read_Write_e.
 * 									  Can be one of the following:
 * 									  @arg @ref I2C_TYPE_WRITE
 *									  @arg @ref I2C_TYPE_READ
 *
 * @param	void : \b *start_callback -  Specifies the callback function which is called once the I2C transaction starts.
 *										 Example:
 *										 @arg @ref &iqs269_close_window - this is used to close virtual comms window
 *					
 *
 * @param	void : \b *trans_callback - Specifies the callback function which is called once transaction is complete, but not end of transaction queue.
 *										must be used with I2C_Last_e action parameter as WAIT.
 *										Example:
 *										@arg @ref &iqs269_trans_callback
 *
 * @param	void : \b *stop_callback - Specifies the callback function which is called if transaction is last in queue. Is called when STOP bit is sent via I2C.
 *									   Example:
 *									   @arg @ref &iqs269_end_job - this is used to indicate transaction queue (job) is done.
 *
 * @param	I2C_Last_e : \b action - Specifies the action to be taken once transaction is complete. 
 *									 Can be one of the following:
 *						  			 @arg @ref I2C_LAST_NO_FORCE - No specific action, driver will determine whether to end comms window or not.
 *		   				  			 @arg @ref I2C_LAST_WAIT - Wait after transaction is completed without closing comms window. Use this and call trans_callback to queue more transactions or end comms window.
 *		   				  			 @arg @ref I2C_LAST_FORCE_LAST - End comms window after transaction, irrelevant of what is in the queue.
 *
 * @param	I2C_transaction_key_e : \b key - This key is used to indicate whether transaction must remain in the queue and be executed every comms window,
 *										 	 or whether must be deleted from the queue after being executed once.
 *											 Can be one of the following:
 *		   									 @arg @ref  I2C_TRANS_REPEAT - Keep transaction in queue for every comms window.
 *		   									 @arg @ref  I2C_TRANS_DELETE - remove transaction from queue after executed.
 *
 * @param	I2C_trans_position_e : \b position - This indicates whether transaction must be added to front or back of queue.
 *												 Can be one of the following:
 *		   										 @arg @ref I2C_TRANS_FIRST - add transaction to the front of the queue
 *		   										 @arg @ref I2C_TRANS_LAST - add transaction to the back of the queue
 *
 * @retval \b bool - indicates whether transaction was successfully queued.
 * 
 */
bool queue_iqs_transaction(I2C_TypeDef *I2Cx, uint8_t addr, uint8_t reg, uint8_t *buffer, uint8_t num_bytes, I2C_Read_Write_e type,
		void *start_callback, void *trans_callback, void *stop_callback, I2C_Last_e action, I2C_transaction_key_e key, I2C_trans_position_e position)
{
	/* Variable for temporary transaction data */
	I2C_transaction_s temp_trans;

	/* Check whether queue is empty */
	if(List_getSize((List_t *)&_iqsPtr->iqs269_queue) >= 20)
	{
		return false;
	}

	/* Assign temporary transaction data */
	temp_trans.i2cPeripheral = I2Cx;
	temp_trans.addr = addr;
	temp_trans.reg = reg;
	temp_trans.buffer = buffer;
	temp_trans.num_bytes= num_bytes;
	temp_trans.index = 0;
	temp_trans.trans_type = type;
	temp_trans.trans_last = false;
	temp_trans.trans_complete_action = action;
	temp_trans.I2C_T_State = I2C_TRANS_NONE;
	temp_trans.State = I2C_NONE;
	temp_trans.I2C_Timeout = 100;
	temp_trans.I2C_Timestamp = HAL_GetTick();
	temp_trans.trans_key = key;
	temp_trans.trans_position = position;
	temp_trans._start_callback = start_callback;
	temp_trans._trans_callback = trans_callback;
	temp_trans._stop_callback = stop_callback;

	/* add the transaction to the queue at start or end depending on transaction selection */
	if(temp_trans.trans_position == I2C_TRANS_LAST)
	{
		List_insertEnd(&_iqsPtr->iqs269_queue, temp_trans.trans_key, &temp_trans);
	}
	else if(temp_trans.trans_position == I2C_TRANS_FIRST)
	{
		List_insertStart(&_iqsPtr->iqs269_queue, temp_trans.trans_key, &temp_trans);
	}

	return true;
}

/**
 * @name	iqs269_checkReadyPin
 * 
 * @brief	Function to check state of the Ready Pin
 *
 * @param	None
 *
 * @retval	\b bool - True if Ready is Low, False if Ready is high.
 * 
 */
bool iqs269_checkReadyPin(void)
{
	/* Manually check state of Ready pin using LL Driver function */
	if(!LL_GPIO_IsInputPinSet(_iqsPtr->azq_iqs269.readyPort, _iqsPtr->azq_iqs269.readyPin))
	{
		return true;
	}
	else
	{
		return false;
	}

}

/**
 * @name	iqs269_rdy_callback
 * @brief	IQS269 ready callback function. The function checks the current state of the IQS device as well as whether the correct interrupt has been received. 
 * 		  	The device state is then updated to data ready state.
 * 
 * @param 	uint8_t : \b IR - Interrupt type, options specified in the STM32 LL Driver.
 * 							  Can be one of the following:
 * 							  @arg @ref LL_EXTI_TRIGGER_FALLING
 * 							  @arg @ref LL_EXTI_TRIGGER_RISING
 * 							  @arg @ref LL_EXTI_TRIGGER_RISINGFALLING
 * 
 * @retval	\b bool - indicates whether specific interrupt type was successfully identified. 
 */
bool iqs269_rdy_callback(uint8_t IR)
{
	/* Check whether received intterupt type is Falling */
	if(IR == LL_EXTI_TRIGGER_FALLING && iqs269_checkReadyPin())
	{
		/* Open comms window */
		iqs269_open_window();

		if((_iqsPtr->state == IQS269_STATE_IDLE)
				&& (IR == _iqsPtr->azq_iqs269.Interrupt_Type))
		{
			/* update the iqs269 timestamp */

			_iqsPtr->state = IQS269_STATE_DATA_READY;
	
			iqs269_start_job();
		}

		return true;
	}
		/* Check whether received intterupt type is Rising */
	else if(IR == LL_EXTI_TRIGGER_RISING && !iqs269_checkReadyPin())
	{
		/* Close comms window */
		iqs269_close_window();

		/* End job */
		iqs269_end_job();

		/* Update flag to indicate that the IQS values are updated */
		_iqsPtr->updated = true;

		return true;
	}
	else
	{

		return false;
	}
}


/**
 * @name	iqs269_open_window
 * 
 * @brief	Function used to open the comms window
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_open_window(void)
{
	_iqsPtr->openwindow = true;
}

/**
 * @name	iqs269_close_window
 * 
 * @brief	Function used to close the comms window
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_close_window(void)
{
	_iqsPtr->jobdone = false;
	_iqsPtr->openwindow = false;
}

/**
 * @name	iqs269_start_job
 * 
 * @brief	Function used to indicate that the job queue has started to execute
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_start_job(void)
{
	_iqsPtr->jobdone = false;
}

/**
 * @name	iqs269_end_job
 * 
 * @brief	Function used to indicate that the job queue has finished executing.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_end_job(void)
{
	_iqsPtr->jobdone = true;
}

/**
 * @name	iqs269_trans_callback
 * 
 * @brief	This function is the callback used after a transaction in the job queue.
 * 			This function allows more transactions to be scheduled in the same comms window depending on conditions after certain transactions.
 * 			The commented section inside the function provides the user with an example where more transactions are schedule if a touch state is detected. 
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_trans_callback(void)
{
	/* Below is an example callback function whether the touch state is monitored and the channel counts are also read from the chip if a touch is detected 

	if(((IQS269_CHAN_STATES*)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))->ch0_touch == 1)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.starting_register,
					&((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch0_filt_ct_1,
					2, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE, I2C_TRANS_DELETE, I2C_TRANS_FIRST);

		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_CNT_CH1_L,
					&((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch1_filt_ct_1,
					2, I2C_TYPE_READ, &iqs269_close_window, NULL ,&iqs269_end_job, I2C_LAST_NO_FORCE, I2C_TRANS_DELETE, I2C_TRANS_FIRST);
	}
	*/

}

/**
 * @name	iqs269_service_update_flag
 * 
 * @brief 	Function to service the update flag
 * 
 * @param	None
 * 
 * @retval	\b None
 * 
 */
bool iqs269_service_update_flag(void)
{
	/* return the update flag state and reset it*/
	if(_iqsPtr->updated)
	{
		_iqsPtr->updated = false;
		return true;
	}

	return false;
}

/**
 * @name	iqs269_set_retry_flag
 * 
 * @brief	Function to set the retry flag
 * 
 * @param	None
 *  
 * @retval	\b None
 * 
 */
void iqs269_set_retry_flag(void)
{
	_iqsPtr->retry_flag = true;

}

/**
 * @name	iqs269_setbit_ATI
 * 
 * @brief	This function is used to set the REDO_ATI bit in the _iqsPtr
 * 
 * @param	None
 * 
 * @retval	\b None
 * 
 */
void iqs269_setbit_ATI(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->REDO_ATI = 1;
}

/**
 * @name	iqs269_resetbit_ATI
 * 
 * @brief	This function is used to reset the REDO_ATI bit in the _iqsPtr
 * 
 * @param	None
 * 
 * @retval	\b None
 * 
 */
void iqs269_resetbit_ATI(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->REDO_ATI = 0;
}

/**
 * @name	iqs269_atiChannels 
 * @brief	A function to ATI selected channels
 *
 * @param	uint8_t : \b channelByte - This byte is the reseed byte which indicates which channels must be ATI'd
 * @param	IQS269_StopRestart : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													or whether a restart condition must be generated.
 * 													Can be one of the following:
 * 													@arg @ref I2C_STOP - End communication window once transaction is complete.
 * 													@arg @ref I2C_RESTART - Generate restart condition.
 *
 *
 * @retval	\b bool - indicate ATI command was sent.
 *
 *
 */
bool iqs269_atiChannels(uint8_t channelByte, IQS269_StopRestart_e stopOrRestart)
{
	/* First set the relevant bits in the reseed byte. */
	iqs269_setReseedByte(channelByte);

	/* Queue transaction to write Reseed byte to IQS chip, use callback to set ATI bit once transaction is complete */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_RESEED_CHANNELS,
		&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_swipe_thres_slid,
		2, I2C_TYPE_WRITE, &iqs269_close_window, &iqs269_setbit_ATI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	/* Check for STOP or RESTART condition and queue appropriate transaction. A callback is used to Reset the ATI bit in the _iqsPtr once the ATI command is sent */
	if(stopOrRestart == I2C_STOP)
	{
	
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_resetbit_ATI, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			2, I2C_TYPE_WRITE, &iqs269_close_window, &iqs269_resetbit_ATI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}

  return true;
}


/**
 * @name	iqs269_GestureEvent
 * 
 * @brief	This function returns the state of the global gesture flag of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b bool - True indicates the gesture flag is set, false indicates the flag is reset. 
 * 
 */
bool iqs269_GestureEvent(void)
{

  if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
  								->gesture == 1)
  {
	  return true;
  }
  else
  {
	  return false;
  }
}

/**
 * @name	iqs269_TouchEvent
 * 
 * @brief	This function returns the state of the global touch flag of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b bool - True indicates the global touch flag is set, false indicates the flag is reset. 
 * 
 */
bool iqs269_TouchEvent(void)
{
  if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
  								->touch_event == 1)
  {
	  return true;
  }
  else
  {
	  return false;
  }
}

/**
 * @name	iqs269_ProxEvent
 * 
 * @brief	This function returns the state of the global prox flag of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b bool - True indicates the global prox flag is set, false indicates the flag is reset. 
 * 
 */
bool iqs269_ProxEvent(void)
{
  if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
  								->prox_event == 1)
  {
	  return true;
  }
  else
  {
	  return false;
  }
}


/**
 * @name	iqs269_PM_ChangeEvent
 * 
 * @brief	This function returns the state of the global power mode event flag of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b bool - True indicates the global power mode event flag is set, false indicates the flag is reset. 
 * 
 */
bool iqs269_PM_ChangeEvent(void)
{
  if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
  								->power_mode_event == 1)
  {
	  return true;
  }
  else
  {
	  return false;
  }
}

/**
 * @name	iqs269_DeepTouchEvent
 * 
 * @brief	This function returns the state of the global DeepTouch flag of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b bool - True indicates the global DeepTouch flag is set, false indicates the flag is reset. 
 * 
 */
bool iqs269_DeepTouchEvent(void)
{

  if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
  								->deep_touch_event == 1)
  {
	  return true;
  }
  else
  {
	  return false;
  }
}


/**
 * @name	iqs269_GetPowerMode
 * 
 * @brief	This function returns the current power mode of the iqs269
 * 
 * @param	None
 * 
 * @retval	\b IQS269_PowerMode_e - Returns the current power mode of the iqs269
 * 			Can be one of the following:
 * 			@arg @ref	IQS269_POWER_MODE_NORMAL 	- Normal Power
 * 			@arg @ref	IQS269_POWER_MODE_LOW	 	- Low Power
 * 			@arg @ref	IQS269_POWER_MODE_ULTRA_LOW - Ultra Low Power
 * 			@arg @ref	IQS269_POWER_MODE_HALT		- Halt Mode
 * 
 */
IQS269_PowerMode_e iqs269_GetPowerMode(void)
{
	  switch(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))->Power_mode)
	  {
	      case IQS269_POWER_MODE_NORMAL:

	    	  return IQS269_POWER_MODE_NORMAL;

		  break;

	      case IQS269_POWER_MODE_LOW:

	    	  return IQS269_POWER_MODE_LOW;

	      case IQS269_POWER_MODE_ULTRA_LOW:

	    	  return IQS269_POWER_MODE_ULTRA_LOW;

	      case IQS269_POWER_MODE_HALT:

	    	  return IQS269_POWER_MODE_HALT;
	  }

}

/**
 * @name	iqs269_setbit_16MHzOSC
 * 
 * @brief	Function to reset the Main Oscillator bit in the _iqsPtr. This will activate the 16Mhz clock. 
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_16MHzOSC(void)
{
	/* Set the appropriate clock selection bit in the local Memory Map */
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->Main_Osc = 0;
}


/**
 * @name	iqs269_setbit_4MHzOSC
 * 
 * @brief	Function to set the Main Oscillator bit in the _iqsPtr. This will activate the 4Mhz clock. 
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_4MHzOSC(void)
{
	/* Set the appropriate clock selection bit in the local Memory Map */
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->Main_Osc = 1;
}

/**
 * @name	iqs269_setFOSC
 * @brief	This function is used to set the Main Oscillator speed of the IQS269A.
 *
 *
 * @param	IQS269_SystemClock_e : System_ClockSpeed - Specifies which oscillator speed must be selected from the the IQS269A_SystemClock_e
 * 		  	Can be one of the following:
 * 		  	@arg @ref IQS269_SYSTEM_CLOCK_4MHZ - Select 4MHz Oscillator Speed
 * 		  	@arg @ref IQS269_SYSTEM_CLOCK_16MHZ - Select 16MHz Oscillator Speed
 *
 * @param	IQS269_StopRestart : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													or whether a restart condition must be generated.
 * 													Can be one of the following:
 * 													@arg @ref I2C_STOP - End communication window once transaction is complete.
 * 													@arg @ref I2C_RESTART - Generate restart condition.
 *
 *
 * @retval	\b bool - indicates whether the command was successfully sent to the I2C queue.
 *
 */
bool iqs269_setFOSC(IQS269_SystemClock_e System_ClockSpeed, IQS269_StopRestart_e stopOrRestart)
{

	/* Update the PMU Settings register in the _iqsPtr from the chip 
		use appropriate callback function to set bit for setting change */

	switch(System_ClockSpeed)
	{
		case IQS269_SYSTEM_CLOCK_4MHZ:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_4MHzOSC, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;



		case IQS269_SYSTEM_CLOCK_16MHZ:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_16MHzOSC, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

	}

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated and queue appropriate transaction. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}

		return true;

}

/**
 * @name	iqs269_setbit_NPMode
 * @brief	This function is used to set the Power Mode Select 3-bit value to Normal Power in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_NPMode(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
										->PM_Select = 0;
}

/**
 * @name	iqs269_setbit_LPMode
 * @brief	This function is used to set the Power Mode Select 3-bit value to Low Power in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_LPMode(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->PM_Select = 1;
}


/**
 * @name	iqs269_setbit_ULPMode
 * @brief	This function is used to set the Power Mode Select 3-bit value to Ultra Low Power in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_ULPMode(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->PM_Select = 2;
}

/**
 * @name	iqs269_setbit_HALTMode
 * @brief	This function is used to set the Power Mode Select 3-bit value to HALT Mode in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_HALTMode(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->PM_Select = 3;
}


/**
 * @name	iqs269_setPowerMode
 * @brief	This function is used to set the Power Mode of the IQS269
 *
 *
 * @param	IQS269_PowerMode_e : \b PowerMode - Specifies the power mode in which the IQS269 must be placed. 
 * 								 Can be one of the following:
 * 								 @arg @ref	IQS269_POWER_MODE_NORMAL 	- Normal Power
 * 								 @arg @ref	IQS269_POWER_MODE_LOW	 	- Low Power
 * 								 @arg @ref	IQS269_POWER_MODE_ULTRA_LOW - Ultra Low Power
 * 								 @arg @ref	IQS269_POWER_MODE_HALT		- Halt Mode
 *
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 *
 *
 * @retval	\b bool - indicates whether the command was successfully sent to the I2C queue.
 *
 */
void iqs269_setPowerMode(IQS269_PowerMode_e PowerMode, IQS269_StopRestart_e stopOrRestart)
{

	/* Update the appropriate settings from the iqs269 into the _iqsPtr and use callback to set desired power mode bits */

	switch(PowerMode)
	{
		case IQS269_POWER_MODE_NORMAL:
			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				2, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_NPMode, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case IQS269_POWER_MODE_LOW:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				2, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_LPMode, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case IQS269_POWER_MODE_ULTRA_LOW:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				2, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_ULPMode, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
		break;

		case IQS269_POWER_MODE_HALT:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
				&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
				2, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_HALTMode, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

	}


	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}



}


/** 
 * @name		iqs269_readChannelCounts
 * @brief		A function which reads and returns the counts value of a specific channel from the _iqsPtr. 
 * 
 * @details		The two bytes containing the counts value are constructed into
 * 				the 16 bit counts value before returning.
 * @param		IQS269_Channel_e	:	\b channel - The channel which the counts must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval		\b uint16_t	- containing the counts value of the channel.
 * @retval		\b 0x0000	- if an invalid channel is selected.
 */
uint16_t iqs269_readChannelCounts(IQS269_Channel_e channel)
{
	uint16_t returnValue;

	  switch(channel)
	  {
	  	  case IQS269_CH0:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch0_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch0_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH1:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch1_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch1_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH2:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch2_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch2_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH3:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch3_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch3_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH4:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch4_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch4_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH5:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch5_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch5_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH6:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch6_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch6_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH7:
	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch7_filt_ct_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch7_filt_ct_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  default:
	  		  return 0x0000;

	  }

}


/** 
 * @name		iqs269_readChannelLta
 * @brief		A function which reads and returns the LTA value of a specific channel from the _iqsPtr. 
 * 
 * @details		The two bytes containing the counts value are constructed into
 * 				the 16 bit counts value before returning.
 * @param		IQS269_Channel_e	:	\b channel - The channel which the LTA must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval		\b uint16_t	- containing the LTA value of the channel.
 * @retval		\b 0x0000	- if an invalid channel is selected.
 */
uint16_t iqs269_readChannelLta(IQS269_Channel_e channel)
{
	uint16_t returnValue;


	  switch(channel)
	  {
	  	  case IQS269_CH0:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch0_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch0_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH1:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch1_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch1_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH2:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch2_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch2_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH3:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch3_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch3_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH4:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch4_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch4_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH5:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch5_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch5_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH6:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch6_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch6_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH7:

	  		  returnValue = (uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch7_lta_1;
	  		  returnValue |= (uint16_t)(((uint16_t)((IQS269_FILTERED_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_filtered_counts_memsec.section_pointer))->iqs269_ch7_lta_2)<<8);
	  		  return returnValue;

	  	  break;

	  	  default:
	  		  return 0x0000;

	  }
}


/** 
 * @name		iqs269_Channel_TouchState
 * @brief		A function which reads and returns the touch state of a specific channel from the _iqsPtr. 
 * 
 *             
 * @param		IQS269_Channel_e	:	\b channel - The channel which the touch state must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval		\b bool	- indicating the touch state of the channel.
 */
bool iqs269_Channel_TouchState(IQS269_Channel_e channel)
{

	  switch(channel)
	  {
	  	  case IQS269_CH0:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch0_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH1:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch1_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH2:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch2_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH3:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch3_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH4:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch4_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH5:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch5_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH6:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch6_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH7:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
											->ch7_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  default:
	  		  return false;

	  }

}

/** 
 * @name		iqs269_Channel_DeepTouchState
 * @brief		A function which reads and returns the deep touch state of a specific channel from the _iqsPtr. 
 * 
 *             
 * @param		IQS269_Channel_e	:	\b channel - The channel which the deep touch state must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval		\b bool	- indicating the deep touch state of the channel.
 * 
 */
bool iqs269_Channel_DeepTouchState(IQS269_Channel_e channel)
{

	  switch(channel)
	  {
	  	  case IQS269_CH0:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch0_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH1:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch1_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH2:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch2_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH3:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch3_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH4:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch4_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH5:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch5_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH6:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch6_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH7:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch7_d_touch == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;



	  	  default:
	  		  return false;

	  }

}

/** 
 * @name 		iqs269_Channel_ProxState
 * @brief      A function which reads and returns the Prox state of a specific channel from the _iqsPtr. 
 *             
 * @param		IQS269_Channel_e	:	\b channel - The channel which the prox state must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval     \b bool	- indicating the prox state of the channel.
 */
bool iqs269_Channel_ProxState(IQS269_Channel_e channel)
{

	  switch(channel)
	  {
	  	  case IQS269_CH0:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch0_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH1:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch1_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH2:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch2_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH3:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch3_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH4:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch4_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH5:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch5_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH6:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch6_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH7:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch7_prox == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;



	  	  default:
	  		  return false;

	  }

}

/** 
 * @name 		iqs269_Channel_ProxDIState
 * @brief      A function which reads and returns the touch state of a specific channel from the _iqsPtr. 
 * 
 *             
 * @param		IQS269_Channel_e	:	\b channel - The channel which the prox direction state must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval     \b bool	- indicating the Prox Direction state of the channel.
 */
bool iqs269_Channel_ProxDIState(IQS269_Channel_e channel)
{
	  switch(channel)
	  {
	  	  case IQS269_CH0:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch0_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH1:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch1_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH2:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch2_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH3:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch3_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH4:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch4_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH5:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch5_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH6:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch6_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;

	  	  case IQS269_CH7:
	  		  if(((IQS269_CHAN_STATES *)(_iqsPtr->iqs269_memsec.iqs269_chan_states_memsec.section_pointer))
										->ch7_prox_di == 1)
	  		  {
	  			  return true;
	  		  }
	  		  else
	  		  {
	  			  return false;
	  		  }
	  	  break;



	  	  default:
	  		  return false;

	  }
}

/** 
 * @name	iqs269_SliderEvent
 * 
 * @brief	A function which returns the Gesture event of the selected slider from the _iqsPtr. 
 * 
 * @param	IQS269_Slider_e : \b slider -	The slider which the gesture event must be read from.
 *                           				Use the IQS269_Slider_e enum defined in iqs269_typedefs.h.
 * 											Can be one of the following:
 * 											@arg @ref IQS269_SL0
 * 											@arg @ref IQS269_SL1
 * 
 * @retval	\b IQS269_gesture_e - enum indicating which gesture event was returned from the selected slider. Default return is NO_SLIDER_SELECTED. 
 * 								  Full list of return options can be seen in IQS269_gesture_e enum defined in iqs269_typedefs.h. 
 * 
 */
IQS269_gesture_e iqs269_SliderEvent(IQS269_Slider_e slider)
{
	switch(slider)
	{
		case IQS269_SL0:
			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->flick_neg_0 == 1)
			{
				return FLICK_NEG_0;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->flick_pos_0 == 1)
			{
				return FLICK_POS_0;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->hold_0 == 1)
			{
				return HOLD_0;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->tap_0 == 1)
			{
				return TAP_0;
			}
			else
			{
				return NO_GESTURE_0;
			}



		break;

		case IQS269_SL1:

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->flick_neg_1 == 1)
			{
				return FLICK_NEG_1;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->flick_pos_1 == 1)
			{
				return FLICK_POS_1;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->hold_1 == 1)
			{
				return HOLD_1;
			}

			if(((IQS269_GESTURE_EV *)(_iqsPtr->iqs269_memsec.iqs269_gesture_ev_memsec.section_pointer))
										->tap_1 == 1)
			{
				return TAP_1;
			}

			else
			{
				return NO_GESTURE_1;
			}

		break;

	  	  default:
	  		  return NO_SLIDER_SELECTED;


	}
}

/**
 * @name	iqs269_SliderCoordinates
 * 
 * @brief	A function which returns the slider coordinates of the selected slider.
 * 
 * @param	IQS269_Slider_e	: \b slider	-	The slider which the coordinates must be read from.
 *                           				Use the IQS269_Slider_e enum defined in iqs269_typedefs.h.
 * 
 * @retval	\b uint8_t	-	returning the coordinates of selected slider. Default return is 0x00.
 * 
 */
uint8_t iqs269_SliderCoordinates(IQS269_Slider_e slider)
{
	switch(slider)
	{
		case IQS269_SL0:

			return ((IQS269_SLIDER_CO *)(_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.section_pointer))->iqs269_slid0_co;

		break;


		case IQS269_SL1:

			return ((IQS269_SLIDER_CO *)(_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.section_pointer))->iqs269_slid1_co;

		break;

		default:
			return 0x00;

	}

}


/** 
 * @name		iqs269_readChannelDelta
 * 
 * @brief		A function which returns the Delta value of a
 *             	specific channel.
 * 
 * @details    	The two bytes containing the delta value are constructed into
 *             	the 16 bit delta value before returning.
 * 
 * @param		IQS269_Channel_e	:	\b channel - The channel which the prox direction state must be read from. IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * @retval     	\b int16_t  - containing the delta value of the channel.
 * @retval     	\b 0x0000	- if an invalid channel is selected.
 * 
 */
int16_t iqs269_readChannelDelta(IQS269_Channel_e channel)
{
	int16_t returnValue;


	  switch(channel)
	  {
	  	  case IQS269_CH0:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch0_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch0_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH1:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch1_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch1_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH2:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch2_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch2_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH3:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch3_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch3_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH4:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch4_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch4_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH5:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch5_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch5_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH6:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch6_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch6_dta_ct_2<<8));

	  		  return returnValue;

	  	  break;

	  	  case IQS269_CH7:

	  		  returnValue = (int16_t)((((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch7_dta_ct_1)|
	  				  		(((IQS269_DELTA_COUNTS *)(_iqsPtr->iqs269_memsec.iqs269_dta_memsec.section_pointer))->iqs269_ch7_dta_ct_2<<8));

	  		  return returnValue;
				
	  	  break;



	  	  default:
	  		  return 0x0000;

	  }
}

/**
 * @name	iqs269_setbit_AutoPSwitch
 * @brief	This function is used to set the Auto Power Switch bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_AutoPSwitch(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->Auto_PM_Switch = 1;

}

/**
 * @name	iqs269_setbit_AutoPSwitch
 * @brief	This function is used to reset the Auto Power Switch bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_resetbit_AutoPSwitch(void)
{

	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->Auto_PM_Switch = 0;;
}


/**
 * @name	iqs269_enable_AutoPower_switch
 * @brief	This function is used to enable Auto Power Mode Switching on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_enable_AutoPower_switch(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_resetbit_AutoPSwitch for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
		&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_resetbit_AutoPSwitch, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
}


/**
 * @name	iqs269_disable_AutoPower_switch
 * @brief	This function is used to disable Auto Power Mode Switching on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_disable_AutoPower_switch(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_setbit_AutoPSwitch for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
		&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_AutoPSwitch, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);


	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.starting_register,
			&((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))->buffer[0],
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
}


/**
 * @name	iqs269_getProdNumber
 * @brief	This function is used to retrieve the product number from the _iqsPtr. 
 *
 * @param	None
 *
 * @retval	\b uint8_t	-	Product Number in _iqsPtr.  
 *
 */
uint8_t iqs269_getProdNumber(void)
{
	return ((IQS269_PROD_INFO *)(_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.section_pointer))->iqs269_prod_num;
}


/**
 * @name	iqs269_getSoftVersion
 * @brief	This function is used to retrieve the software number from the _iqsPtr. 
 *
 * @param	None
 *
 * @retval	\b uint8_t	-	Software Number in _iqsPtr.  
 *
 */
uint8_t iqs269_getSoftVersion(void)
{
	return ((IQS269_PROD_INFO *)(_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.section_pointer))->iqs269_soft_ver;
}

/**
 * @name	iqs269_getHardNumber
 * @brief	This function is used to retrieve the hardware number from the _iqsPtr. 
 *
 * @param	None
 *
 * @retval	\b uint8_t	-	Hardware Number in _iqsPtr.  
 *
 */
uint8_t iqs269_getHardNumber(void)
{
	return ((IQS269_PROD_INFO *)(_iqsPtr->iqs269_memsec.iqs269_prod_info_memsec.section_pointer))->iqs269_hard_num;
}

/**
 * @name	iqs269_setbit_enableHallUI
 * @brief	This function is used to set the Enable Hall UI bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableHallUI(void)
{
	((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))
								->hall_ui_enable = 1;
}

/**
 * @name	iqs269_resetbit_enableHallUI
 * @brief	This function is used to reset the Enable Hall UI bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_resetbit_enableHallUI(void)
{
	((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))
								->hall_ui_enable = 0;
}

/**
 * @name	iqs269_enableHallUI
 * @brief	This function is used to enable the HALL UI on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_enableHallUI(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_resetbit_enableHallUI for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_HALL_UI_ENABLE,
		&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable, 
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_resetbit_enableHallUI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_HALL_UI_ENABLE,
			&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_HALL_UI_ENABLE,
			&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}	
}


/**
 * @name	iqs269_disableHallUI
 * @brief	This function is used to disable the HALL UI on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_disableHallUI(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_setbit_enableHallUI for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_HALL_UI_ENABLE,
		&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable, 
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableHallUI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_HALL_UI_ENABLE,
			&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_HALL_UI_ENABLE,
			&((IQS269_HALL_UI_ENABLE *)(_iqsPtr->iqs269_memsec.iqs269_hall_ui_en_memsec.section_pointer))->iqs269_hall_ui_enable,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
}

/**
 * @name	iqs269_setbit_enableAbsCapUI
 * @brief	This function is used to set the Enable ABS CAP UI bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapUI(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_enable = 1;
}

/**
 * @name	iqs269_setbit_disableAbsCapUI
 * @brief	This function is used to set the Disable ABS CAP UI bit in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_disableAbsCapUI(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_enable = 0;

}

/**
 * @name	iqs269_enableAbsCapUI
 * @brief	This function is used to enable Abs Cap UI on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_enableAbsCapUI(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_setbit_enableAbsCapUI for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_ABS_CAP_SETTINGS,
		&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapUI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);


	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	}


}

/**
 * @name	iqs269_disableAbsCapUI
 * @brief	This function is used to disable Abs Cap UI on the iqs269.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_disableAbsCapUI(IQS269_StopRestart_e stopOrRestart)
{
	/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_setbit_disableAbsCapUI for appropriate bit update.  */
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_ABS_CAP_SETTINGS,
		&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
		1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_disableAbsCapUI, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);


	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	}
}


/**
 * @name	iqs269_abscap_MeasCap_Selection
 * @brief	This function is used to set the Absolute Capacitance Settings.
 * 
 * @param	IQS269_Cap_Meas_e	:	\b Cx	- Specifies the Absolute Capacitance Measurement Capacitor
 * 									Can be one of the following:
 * 									@arg @ref IQS269_C0
 * 									@arg @ref IQS269_C1
 * 
 * @param	IQS269_Fill_Cap_e	:	\b C_fill	-	Specifies the Absolute Capacitance Filling Cap Size
 * 									Can be one of the following:
 *   								@arg @ref IQS269_0pF5
 *									@arg @ref IQS269_1pF0
 *									@arg @ref IQS269_1pF5
 *									@arg @ref IQS269_2pF0
 *
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_abscap_MeasCap_Selection(IQS269_Cap_Meas_e Cx, IQS269_Fill_Cap_e C_fill, IQS269_StopRestart_e stopOrRestart)
{
	/* Switch case to check which Measurement Cap was selected */
	switch(Cx)
	{
		case IQS269_C0:
			/* Switch case to check which filling cap was selected. The _iqsPtr settings is updated from te chip.
				The appropriate callback is used to set the appropriate bit */
			switch(C_fill)
			{
				case IQS269_0pF5:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap1_1,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

				case IQS269_1pF0:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap1_2,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

				case IQS269_1pF5:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap1_3,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);


				break;

				case IQS269_2pF0:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap1_4,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);


				break;
			}
		break;

		case IQS269_C1:
			switch(C_fill)
			{
				case IQS269_0pF5:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap2_1,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

				case IQS269_1pF0:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap2_2,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

				case IQS269_1pF5:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap2_3,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

				case IQS269_2pF0:

					queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
						IQS269_MM_ABS_CAP_SETTINGS,
						&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->
						iqs269_abs_cap_settings, 1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_abscapRefCap2_4,
						&iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;
			}
		break;
	}

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	}



}

/**
 * @name	iqs269_QueueABSCapValueUpdates
 * @brief	This function is used to queue the value updates to be used in ABS Cap Mode
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_QueueABSCapValueUpdates(void)
{
	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_SYSTEM_FLAGS,
		&((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))->buffer[0],
		10, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_NO_FORCE, I2C_TRANS_REPEAT, I2C_TRANS_LAST);



	queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
		IQS269_MM_COORDINATE_SLDR0,
		&((IQS269_SLIDER_CO *)(_iqsPtr->iqs269_memsec.iqs269_slider_co_memsec.section_pointer))->buffer[0],
		10, I2C_TYPE_READ, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_REPEAT, I2C_TRANS_LAST);
}


/**
 * @name	iqs269_setbit_abscapRefCap1_1
 * @brief	This function is used to set the bits for the 0.5pF Fill Cap option for the C0 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap1_1(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap0_sel = 0;
}


/**
 * @name	iqs269_setbit_abscapRefCap1_2
 * @brief	This function is used to set the bits for the 1pF Fill Cap option for the C0 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap1_2(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap0_sel = 1;
}

/**
 * @name	iqs269_setbit_abscapRefCap1_3
 * @brief	This function is used to set the bits for the 1.5pF Fill Cap option for the C0 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap1_3(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap0_sel = 2;
}

/**
 * @name	iqs269_setbit_abscapRefCap1_4
 * @brief	This function is used to set the bits for the 2pF Fill Cap option for the C0 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap1_4(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap0_sel = 3;
}

/**
 * @name	iqs269_setbit_abscapRefCap2_1
 * @brief	This function is used to set the bits for the 0.5pF Fill Cap option for the C1 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap2_1(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap1_sel = 0;
}

/**
 * @name	iqs269_setbit_abscapRefCap2_2
 * @brief	This function is used to set the bits for the 1pF Fill Cap option for the C1 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap2_2(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap1_sel = 1;
}

/**
 * @name	iqs269_setbit_abscapRefCap2_3
 * @brief	This function is used to set the bits for the 1.5pF Fill Cap option for the C1 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap2_3(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap1_sel = 2;
}

/**
 * @name	iqs269_setbit_abscapRefCap2_4
 * @brief	This function is used to set the bits for the 2pF Fill Cap option for the C1 ABS Cap measurement Capacitor in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_abscapRefCap2_4(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_refcap1_sel = 3;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH0
 * @brief	This function is used to set the bits to enable CH0 as the active ABS Cap Channel in the _iqsPtr. 
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH0(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 0;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH1
 * @brief	This function is used to set the bits to enable CH1 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH1(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 1;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH2
 * @brief	This function is used to set the bits to enable CH2 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH2(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 2;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH3
 * @brief	This function is used to set the bits to enable CH3 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH3(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 3;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH4
 * @brief	This function is used to set the bits to enable CH4 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH4(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 4;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH5
 * @brief	This function is used to set the bits to enable CH5 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH5(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 5;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH6
 * @brief	This function is used to set the bits to enable CH6 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH6(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 6;
}

/**
 * @name	iqs269_setbit_enableAbsCapCH7
 * @brief	This function is used to set the bits to enable CH7 as the active ABS Cap Channel in the _iqsPtr.
 *
 * @param	None
 *
 * @retval	\b None
 *
 */
void iqs269_setbit_enableAbsCapCH7(void)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->abscap_ui_channel_selection = 7;
}


/**
 * @name	iqs269_setAbsCapCH
 * @brief	This function is used to set the channel to be used for the Absolute Capacitance Measurement. 
 * 
 * @param	IQS269_Channel_e	:	\b channel - The channel which must be used for the ABS Cap Measurement IQS269_CH0, IQS269_CH1 etc.
 *                          						 Use the IQS269_Channel_e enum defined in iqs269_typedefs.h to see full list of options.
 * 
 * 
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 * 
 * 
 * @retval	\b None	
 */
void iqs269_setAbsCapCH(IQS269_Channel_e channel, IQS269_StopRestart_e stopOrRestart)
{
	/* Switch case to check which channel was selected for the ABS Cap measurement */
	switch(channel)
	{
		/* Read appropriate settings from iqs269 and update _iqsPtr. Use iqs269_setbit_enableAbsCapCHX for appropriate bit update.  */
		case   IQS269_CH0:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH0, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case  IQS269_CH1:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH1, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case  IQS269_CH2:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH2, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case IQS269_CH3:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH3, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case IQS269_CH4:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH4, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case IQS269_CH5:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH5, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case  IQS269_CH6:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH6, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

		case  IQS269_CH7:

			queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
				IQS269_MM_ABS_CAP_SETTINGS,
				&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
				1, I2C_TYPE_READ, &iqs269_close_window, &iqs269_setbit_enableAbsCapCH7, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

		break;

	}

	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */

	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_ABS_CAP_SETTINGS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_abs_cap_settings,
			1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

	}



}

/**
 * @name	iqs269_getAbsCapMeasurement
 * @brief	This function is used to return the appropriate ABS Cap measurement from the _iqsPtr specified by the Cx parameter. 
 * 
 * @param	IQS269_Cap_Meas_e	:	\b Cx	- Specifies the Absolute Capacitance Measurement Capacitor
 * 									Can be one of the following:
 * 									@arg @ref IQS269_C0
 * 									@arg @ref IQS269_C1
 * 
 * @retval	\b uint32_t	- Returns the absolute capacitance measurement in a uint32_t.  
 * 							The correct value can be obtained by casting to format with decimal and dividing by 100. 
 * 
 */ 
uint32_t iqs269_getAbsCapMeasurement(IQS269_Cap_Meas_e Cx)
{
	uint16_t base;
	uint16_t counts;
	uint16_t counts_Cx;
	uint32_t returnValue;
	uint16_t RefCapSize;
	uint8_t CapSelection1;
	uint8_t CapSelection2;
	uint8_t SoftVersion;

	SoftVersion = iqs269_getSoftVersion();
	/* Calculate correct base value */
	base = (uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_base_l;
	base |= (uint16_t)(((uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_base_h)<<8);

	if(SoftVersion < 0x04)
	{
		base &= ~(0xF800);
	}

	/* Calculate correct counts value */
	counts = (uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_cnt_l;
	counts |= (uint16_t)(((uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_cnt_h)<<8);

	if(SoftVersion < 0x04)
	{
		counts &= ~(0xF800);
	}

	CapSelection1 = ((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->abscap_refcap0_sel;
	CapSelection2 = ((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->abscap_refcap1_sel;

	switch(Cx)
	{
		case IQS269_C0:

			counts_Cx = (uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_calcap0_l;
			counts_Cx |= (uint16_t)(((uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_calcap0_h)<<8);

			if(SoftVersion < 0x04)
			{
				counts_Cx &= ~(0xF800);
			}

			switch(CapSelection1)
			{
				case 0:

					RefCapSize = 5;

				break;

				case 1:

					RefCapSize = 10;

				break;

				case 2:

					RefCapSize = 15;

				break;

				case 3:

					RefCapSize = 20;

				break;
			}

		break;


		case IQS269_C1:
			counts_Cx = (uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_calcap1_l;
			counts_Cx |= (uint16_t)(((uint16_t)((IQS269_ABS_CAP *)(_iqsPtr->iqs269_memsec.iqs269_abs_cap_memsec.section_pointer))->abscap_calcap1_h)<<8);

			if(SoftVersion < 0x04)
			{
				counts_Cx &= ~(0xF800);
			}

			switch(CapSelection2)
			{
				case 0:

					RefCapSize = 5;

				break;

				case 1:

					RefCapSize = 10;

				break;

				case 2:

					RefCapSize = 15;

				break;

				case 3:

					RefCapSize = 20;

				break;
			}


		break;
	}



	/* Calculate Absolute Capacitance, value must be divided by 100 in application to obtain correct measurement with 2 decimals */
	returnValue = (uint32_t)(((uint32_t)RefCapSize*counts*counts_Cx*10)/((uint32_t)(base*(counts-counts_Cx))));

	return returnValue;

}


/** 
 * @name	iqs269_setRefChannel
 * @brief	This function sets the reference channels for a selected channel.
 *
 * @param	IQS269_Channel_e : \b channel - selected channel.
 * @param 	uint8_t : \b ref_channel_byte - byte indicating channels using Channel as reference.
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 *
 * 
 * @retval \b None
 */
void iqs269_setRefChannel(IQS269_Channel_e channel, uint8_t ref_channel_byte, IQS269_StopRestart_e stopOrRestart)
{

	uint8_t channel_reg = IQS269_MM_CH0_FOLLOW_CHANNELS + (channel*7);

	if(stopOrRestart == I2C_STOP)
	{
		switch(channel)
		{
			case IQS269_CH0:

				/* Set reference channel settings for CH0 in _iqsPtr before writing the settings to the chip */
				((IQS269_CH0_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch0 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH0_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch0,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

			case IQS269_CH1:

				((IQS269_CH1_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch1 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH1_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch1_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch1,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH2:

				((IQS269_CH2_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch2 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH2_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch2_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch2,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH3:

				((IQS269_CH3_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch3 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH3_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch3_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch3,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH4:

				((IQS269_CH4_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch4 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH4_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch4_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch4,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH5:

				((IQS269_CH5_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch5 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH5_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch5_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch5,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH6:

				((IQS269_CH6_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch6 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH6_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch6_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch6,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH7:

				((IQS269_CH7_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))
										->iqs269_dcf_chb_association_ch7 = ref_channel_byte;

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH7_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch7_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch7,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

		}
	}
	else if(stopOrRestart == I2C_RESTART)
	{
		switch(channel)
		{
			case IQS269_CH0:
				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH0_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch0_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch0,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

				break;

			case IQS269_CH1:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH1_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch1_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch1,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH2:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH2_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch2_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch2,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH3:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH3_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch3_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch3,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH4:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH4_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch4_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch4,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH5:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH5_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch5_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch5,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH6:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH6_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch6_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch6,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

			case IQS269_CH7:

				queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
					channel_reg,
					&((IQS269_CH7_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_ch7_settings_memsec.section_pointer))->iqs269_dcf_chb_association_ch7,
					1, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);

			break;

		}
	}

}

/** 
 * @name	iqs269_setbit_SoftReset
 * @brief	This function serves as a wrapper function to set the SOFT_RESET bit in the _iqsPtr
 *
 * @param	None 
 *
 * @retval	\b None
 */
void iqs269_setbit_SoftReset(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->SOFT_RESET = 1;
}


/** 
 * @name	iqs269_setbit_SoftReset
 * @brief	This function serves as a wrapper function to reset the SOFT_RESET bit in the _iqsPtr
 *
 * @param	None 
 *
 * @retval	\b None
 */
void iqs269_resetbit_SoftReset(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->SOFT_RESET = 0;
}


/** 
 * @name	iqs269_checkSoftReset
 * @brief	This function serves as a wrapper function to check the Show Reset bit in the _iqsPtr
 *
 * @param	None
 *
 * @retval	\b bool : True indicates bit is set, false indicates the bit is reset. 
 */
bool iqs269_checkSoftReset(void)
{
	if(((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
								->Show_reset == 1)
	{
		return true;
	}
	else
	{
		return false;
	}
}

/** 
 * @name	iqs269_resetbit_ShowReset
 * @brief	This function serves as a wrapper function to reset the Show_reset bit in the _iqsPtr
 *
 * @param	None
 *
 * @retval	\b None
 */
void iqs269_resetbit_ShowReset(void)
{
	((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
								->Show_reset = 0;
}


/** 
 * @name	iqs269_setbit_ShowReset
 * @brief	This function serves as a wrapper function to set the Show_reset bit in the _iqsPtr
 * 
 * @param	None
 *
 * @retval	\b None
 */
void iqs269_setbit_ShowReset(void)
{
	((IQS269_SYS_GE_FLAGS *)(_iqsPtr->iqs269_memsec.iqs269_sys_ge_flags_memsec.section_pointer))
								->Show_reset = 1;
}


/** 
 * @name	iqs269_setbit_AckReset
 * @brief	This function serves as a wrapper function to set the AckReset bit in the _iqsPtr
 *
 * @param	None
 *
 * @retval	\b None
 */
void iqs269_setbit_AckReset(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->ACK_RESET = 1;
}

/** 
 * @name	iqs269_resetbit_AckReset
 * @brief	This function serves as a wrapper function to reset the AckReset bit in the _iqsPtr
 *
 * @param	None
 *
 * @retval	\b None
 */
void iqs269_resetbit_AckReset(void)
{
	((IQS269_POWER_SYS_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_pow_sys_settings_memsec.section_pointer))
									->ACK_RESET = 0;
}

/** 
 * @name	iqs269_setReseedByte
 * @brief	This function sets the reseed enable byte in the _iqsPtr
 *
 * @param	uint8_t	:	\b channelByte	-	This byte indicates which channels must be reseeded. 
 *
 * @retval	\b None
 */
void iqs269_setReseedByte(uint8_t channelByte)
{
	((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))
								->iqs269_chb_reseed_en = channelByte;
}


/** 
 * @name	iqs269_ReseedChannels
 * @brief	This function queues the transaction to reseed the selected channels given a reseed byte as input. 
 *
 * @param	uint8_t	:	\b channelByte	-	This byte indicates which channels must be reseeded. 
 * 
 * @param	IQS269_StopRestart_e : \b stopOrRestart - Specifies whether I2C communication window must be closed once transaction is completed
 * 													  or whether a restart condition must be generated.
 * 								  Can be one of the following:
 * 								  @arg @ref I2C_STOP - End communication window once transaction is complete.
 * 								  @arg @ref I2C_RESTART - Generate restart condition.
 *
 * @retval	\b None
 */
void iqs269_ReseedChannels(uint8_t channelByte, IQS269_StopRestart_e stopOrRestart)
{
	/* Set the Reseed Byte in the _iqsPtr */
	iqs269_setReseedByte(channelByte);


	/* Check whether comms window must be closed once transaction is complete or whether restart condition must be generated 
		and queue appropriate transaction to write settings to chip. */
	if(stopOrRestart == I2C_STOP)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_RESEED_CHANNELS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_swipe_thres_slid,
			2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_FORCE_LAST, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}
	else if (stopOrRestart == I2C_RESTART)
	{
		queue_iqs_transaction(_iqsPtr->azq_iqs269.i2cBus.i2cPeripheral, _iqsPtr->azq_iqs269.i2cAddress,
			IQS269_MM_RESEED_CHANNELS,
			&((IQS269_GLOBAL_SETTINGS *)(_iqsPtr->iqs269_memsec.iqs269_global_settings_memsec.section_pointer))->iqs269_swipe_thres_slid,
			2, I2C_TYPE_WRITE, &iqs269_close_window, NULL, &iqs269_end_job, I2C_LAST_WAIT, I2C_TRANS_DELETE, I2C_TRANS_LAST);
	}

}
