/**
	*****************************************************************************
	* @file			iqs269_addresses.h
	* @ingroup	azq_drivers_iqs269
	* @brief 		IQS269 memory map addresses to be used with IQS269 Driver module.
  * @note     All memory map addresses are preceded with 'IQS269_MM_' to prevent
  *           possible conflicts with other IQS device memory maps.
	*****************************************************************************/
  
#ifndef __IQS269_ADDRESSES_H
#define __IQS269_ADDRESSES_H

/** @name     IQS269 Memory Map
  * @{
  */
/* Device Information - Read Only */
#define IQS269_MM_PRODUCT_NUMBER              0x00  // Byte0
#define IQS269_MM_SOFTWARE_VERSION            0x00  // Byte1
#define IQS269_MM_HWFN                        0x01  // Byte0
#define IQS269_MM_DEVELOPMENT_VERSION         0x01  // Byte1

/* Global Flags - Read Only */
#define IQS269_MM_SYSTEM_FLAGS                0x02  // Byte0
#define IQS269_MM_GLOBAL_EVENTS               0x02  // Byte1
#define IQS269_MM_GESTURE_EVENTS              0x03  // Byte0
#define IQS269_MM_RESERVED0                   0x03  // Byte1
#define IQS269_MM_PROX_STATE                  0x04  // Byte0
#define IQS269_MM_PROX_DIRECTION              0x04  // Byte1
#define IQS269_MM_TOUCH_STATE                 0x05  // Byte0
#define IQS269_MM_DEEP_TOUCH_STATE            0x05  // Byte1
#define IQS269_MM_FOLLOWER_ACTIVE             0x06  // Byte0
#define IQS269_MM_FOLLOWER_MOVEMENT           0x06  // Byte1
#define IQS269_MM_MOVEMENT_DIRECTION          0x07  // Byte0
#define IQS269_MM_MULTI_DIRECTION             0x07  // Byte1

/* Channel Counts and LTA - Read Only */
#define IQS269_MM_CNT_CH0_L                   0x08  // Byte0
#define IQS269_MM_CNT_CH0_H                   0x08  // Byte1
#define IQS269_MM_LTA_CH0_L                   0x09  // Byte0
#define IQS269_MM_LTA_CH0_H                   0x09  // Byte1
#define IQS269_MM_CNT_CH1_L                   0x0A  // Byte0
#define IQS269_MM_CNT_CH1_H                   0x0A  // Byte1
#define IQS269_MM_LTA_CH1_L                   0x0B  // Byte0
#define IQS269_MM_LTA_CH1_H                   0x0B  // Byte1
#define IQS269_MM_CNT_CH2_L                   0x0C  // Byte0
#define IQS269_MM_CNT_CH2_H                   0x0C  // Byte1
#define IQS269_MM_LTA_CH2_L                   0x0D  // Byte0
#define IQS269_MM_LTA_CH2_H                   0x0D  // Byte1
#define IQS269_MM_CNT_CH3_L                   0x0E  // Byte0
#define IQS269_MM_CNT_CH3_H                   0x0E  // Byte1
#define IQS269_MM_LTA_CH3_L                   0x0F  // Byte0
#define IQS269_MM_LTA_CH3_H                   0x0F  // Byte1
#define IQS269_MM_CNT_CH4_L                   0x10  // Byte0
#define IQS269_MM_CNT_CH4_H                   0x10  // Byte1
#define IQS269_MM_LTA_CH4_L                   0x11  // Byte0
#define IQS269_MM_LTA_CH4_H                   0x11  // Byte1
#define IQS269_MM_CNT_CH5_L                   0x12  // Byte0
#define IQS269_MM_CNT_CH5_H                   0x12  // Byte1
#define IQS269_MM_LTA_CH5_L                   0x13  // Byte0
#define IQS269_MM_LTA_CH5_H                   0x13  // Byte1
#define IQS269_MM_CNT_CH6_L                   0x14  // Byte0
#define IQS269_MM_CNT_CH6_H                   0x14  // Byte1
#define IQS269_MM_LTA_CH6_L                   0x15  // Byte0
#define IQS269_MM_LTA_CH6_H                   0x15  // Byte1
#define IQS269_MM_CNT_CH7_L                   0x16  // Byte0
#define IQS269_MM_CNT_CH7_H                   0x16  // Byte1
#define IQS269_MM_LTA_CH7_L                   0x17  // Byte0
#define IQS269_MM_LTA_CH7_H                   0x17  // Byte1

/* Channel Delta Values -Read Only */
#define IQS269_MM_CH0_DTA_L                   0x18  // Byte0
#define IQS269_MM_CH0_DTA_H                   0x18  // Byte1
#define IQS269_MM_CH1_DTA_L                   0x19  // Byte0
#define IQS269_MM_CH1_DTA_H                   0x19  // Byte1
#define IQS269_MM_CH2_DTA_L                   0x1A  // Byte0
#define IQS269_MM_CH2_DTA_H                   0x1A  // Byte1
#define IQS269_MM_CH3_DTA_L                   0x1B  // Byte0
#define IQS269_MM_CH3_DTA_H                   0x1B  // Byte1
#define IQS269_MM_CH4_DTA_L                   0x1C  // Byte0
#define IQS269_MM_CH4_DTA_H                   0x1C  // Byte1
#define IQS269_MM_CH5_DTA_L                   0x1D  // Byte0
#define IQS269_MM_CH5_DTA_H                   0x1D  // Byte1
#define IQS269_MM_CH6_DTA_L                   0x1E  // Byte0
#define IQS269_MM_CH6_DTA_H                   0x1E  // Byte1
#define IQS269_MM_CH7_DTA_L                   0x1F  // Byte0
#define IQS269_MM_CH7_DTA_H                   0x1F  // Byte1

/* Follower UI - Read Only */
#define IQS269_MM_DCF_DTA_CH0_L               0x20  // Byte0
#define IQS269_MM_DCF_DTA_CH0_H               0x20  // Byte1
#define IQS269_MM_DCF_DTA_CH1_L               0x21  // Byte0
#define IQS269_MM_DCF_DTA_CH1_H               0x21  // Byte1
#define IQS269_MM_DCF_DTA_CH2_L               0x22  // Byte0
#define IQS269_MM_DCF_DTA_CH2_H               0x22  // Byte1
#define IQS269_MM_DCF_DTA_CH3_L               0x23  // Byte0
#define IQS269_MM_DCF_DTA_CH3_H               0x23  // Byte1
#define IQS269_MM_DCF_DTA_CH4_L               0x24  // Byte0
#define IQS269_MM_DCF_DTA_CH4_H               0x24  // Byte1
#define IQS269_MM_DCF_DTA_CH5_L               0x25  // Byte0
#define IQS269_MM_DCF_DTA_CH5_H               0x25  // Byte1
#define IQS269_MM_DCF_DTA_CH6_L               0x26  // Byte0
#define IQS269_MM_DCF_DTA_CH6_H               0x26  // Byte1
#define IQS269_MM_DCF_DTA_CH7_L               0x27  // Byte0
#define IQS269_MM_DCF_DTA_CH7_H               0x27  // Byte1

/* Movement UI - Read Only */
#define IQS269_MM_MOV_CNTR_CH0                0x28  // Byte0
#define IQS269_MM_CNTR_TMR_CH0                0x28  // Byte1
#define IQS269_MM_MOV_CNTR_CH1                0x29  // Byte0
#define IQS269_MM_CNTR_TMR_CH1                0x29  // Byte1
#define IQS269_MM_MOV_CNTR_CH2                0x2A  // Byte0
#define IQS269_MM_CNTR_TMR_CH2                0x2A  // Byte1
#define IQS269_MM_MOV_CNTR_CH3                0x2B  // Byte0
#define IQS269_MM_CNTR_TMR_CH3                0x2B  // Byte1
#define IQS269_MM_MOV_CNTR_CH4                0x2C  // Byte0
#define IQS269_MM_CNTR_TMR_CH4                0x2C  // Byte1
#define IQS269_MM_MOV_CNTR_CH5                0x2D  // Byte0
#define IQS269_MM_CNTR_TMR_CH5                0x2D  // Byte1
#define IQS269_MM_MOV_CNTR_CH6                0x2E  // Byte0
#define IQS269_MM_CNTR_TMR_CH6                0x2E  // Byte1
#define IQS269_MM_MOV_CNTR_CH7                0x2F  // Byte0
#define IQS269_MM_CNTR_TMR_CH7                0x2F  // Byte1

/* Slider UI - Read Only */
#define IQS269_MM_COORDINATE_SLDR0            0x30  // Byte0
#define IQS269_MM_COORDINATE_SLDR1            0x30  // Byte1

/* Absolute Capacitance UI - Read Only */
#define IQS269_MM_ABSCAP_BASE_L               0x31  // Byte0
#define IQS269_MM_ABSCAP_BASE_H               0x31  // Byte1
#define IQS269_MM_ABSCAP_CNT_L                0x32  // Byte0
#define IQS269_MM_ABSCAP_CNT_H                0x32  // Byte1
#define IQS269_MM_ABSCAP_CALCAP0_L            0x33  // Byte0
#define IQS269_MM_ABSCAP_CALCAP0_H            0x33  // Byte1
#define IQS269_MM_ABSCAP_CALCAP1_L            0x34  // Byte0
#define IQS269_MM_ABSCAP_CALCAP1_H            0x34  // Byte1

/* Calibration Values - Read Only */
#define IQS269_MM_CALIB_VALUE0_L              0x35  // Byte0
#define IQS269_MM_CALIB_VALUE0_H              0x35  // Byte1
#define IQS269_MM_CALIB_VALUE1_L              0x36  // Byte0
#define IQS269_MM_CALIB_VALUE1_H              0x36  // Byte1

/* Device Settings - Read/Write = 24 bytes*/
#define IQS269_MM_PMU_SETTINGS                0x80  // Byte0
#define IQS269_MM_I2C_SETTINGS                0x80  // Byte1
#define IQS269_MM_ACTIVE_CHANNELS             0x81  // Byte0
#define IQS269_MM_FILTER_SETTINGS             0x81  // Byte1
#define IQS269_MM_RESEED_ENABLE               0x82  // Byte0
#define IQS269_MM_GLOBAL_EVENTS_MASK          0x82  // Byte1
#define IQS269_MM_REPORT_RATE_NP              0x83  // Byte0
#define IQS269_MM_REPORT_RATE_LP              0x83  // Byte1
#define IQS269_MM_REPORT_RATE_ULP             0x84  // Byte0
#define IQS269_MM_MODE_TIMEOUT                0x84  // Byte1
#define IQS269_MM_WINDOW_TIMEOUT              0x85  // Byte0
#define IQS269_MM_HALT_TIMEOUT                0x85  // Byte1
#define IQS269_MM_PXS_GLBL_SETTINGS_0         0x86  // Byte0
#define IQS269_MM_PXS_GLBL_SETTINGS_1         0x86  // Byte1
#define IQS269_MM_ABS_CAP_SETTINGS            0x87  // Byte0
#define IQS269_MM_UI_GLBL_SETTINGS            0x87  // Byte1
#define IQS269_MM_BLOCK_FOLLOW_EVENT          0x88  // Byte0
#define IQS269_MM_RESERVED1                   0x88  // Byte1
#define IQS269_MM_SLIDER0_CHANNELS            0x89  // Byte0
#define IQS269_MM_SLIDER1_CHANNELS            0x89  // Byte1
#define IQS269_MM_TAP_TIMEOUT                 0x8A  // Byte0
#define IQS269_MM_SWIPE_TIMEOUT               0x8A  // Byte1
#define IQS269_MM_GESTURE_THRESHOLD           0x8B  // Byte0
#define IQS269_MM_RESEED_CHANNELS             0x8B  // Byte1

/* Channel 0 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH0_SELECT_CRX              0x8C  // Byte0
#define IQS269_MM_CH0_SELECT_CTX              0x8C  // Byte1
#define IQS269_MM_CH0_PROXCTL                 0x8D  // Byte0
#define IQS269_MM_CH0_PROXCG0                 0x8D  // Byte1
#define IQS269_MM_CH0_PROXCFG1                0x8E  // Byte0
#define IQS269_MM_CH0_ATI_TARGETS             0x8E  // Byte1
#define IQS269_MM_CH0_MULTIPLIERS             0x8F  // Byte0
#define IQS269_MM_CH0_PCC                     0x8F  // Byte1
#define IQS269_MM_CH0_THRESHOLD_P             0x90  // Byte0
#define IQS269_MM_CH0_THRESHOLD_T             0x90  // Byte1
#define IQS269_MM_CH0_THRESHOLD_DT            0x91  // Byte0
#define IQS269_MM_CH0_HYSTERESIS              0x91  // Byte1
#define IQS269_MM_CH0_FOLLOW_CHANNELS         0x92  // Byte0
#define IQS269_MM_CH0_FOLLOW_WEIGHT           0x92  // Byte1

/* Channel 1 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH1_SELECT_CRX              0x93  // Byte0
#define IQS269_MM_CH1_SELECT_CTX              0x93  // Byte1
#define IQS269_MM_CH1_PROXCTL                 0x94  // Byte0
#define IQS269_MM_CH1_PROXCG0                 0x94  // Byte1
#define IQS269_MM_CH1_PROXCFG1                0x95  // Byte0
#define IQS269_MM_CH1_ATI_TARGETS             0x95  // Byte1
#define IQS269_MM_CH1_MULTIPLIERS             0x96  // Byte0
#define IQS269_MM_CH1_PCC                     0x96  // Byte1
#define IQS269_MM_CH1_THRESHOLD_P             0x97  // Byte0
#define IQS269_MM_CH1_THRESHOLD_T             0x97  // Byte1
#define IQS269_MM_CH1_THRESHOLD_DT            0x98  // Byte0
#define IQS269_MM_CH1_HYSTERESIS              0x98  // Byte1
#define IQS269_MM_CH1_FOLLOW_CHANNELS         0x99  // Byte0
#define IQS269_MM_CH1_FOLLOW_WEIGHT           0x99  // Byte1

/* Channel 2 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH2_SELECT_CRX              0x9A  // Byte0
#define IQS269_MM_CH2_SELECT_CTX              0x9A  // Byte1
#define IQS269_MM_CH2_PROXCTL                 0x9B  // Byte0
#define IQS269_MM_CH2_PROXCG0                 0x9B  // Byte1
#define IQS269_MM_CH2_PROXCFG1                0x9C  // Byte0
#define IQS269_MM_CH2_ATI_TARGETS             0x9C  // Byte1
#define IQS269_MM_CH2_MULTIPLIERS             0x9D  // Byte0
#define IQS269_MM_CH2_PCC                     0x9D  // Byte1
#define IQS269_MM_CH2_THRESHOLD_P             0x9E  // Byte0
#define IQS269_MM_CH2_THRESHOLD_T             0x9E  // Byte1
#define IQS269_MM_CH2_THRESHOLD_DT            0x9F  // Byte0
#define IQS269_MM_CH2_HYSTERESIS              0x9F  // Byte1
#define IQS269_MM_CH2_FOLLOW_CHANNELS         0xA0  // Byte0
#define IQS269_MM_CH2_FOLLOW_WEIGHT           0xA0  // Byte1

/* Channel 3 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH3_SELECT_CRX              0xA1  // Byte0
#define IQS269_MM_CH3_SELECT_CTX              0xA1  // Byte1
#define IQS269_MM_CH3_PROXCTL                 0xA2  // Byte0
#define IQS269_MM_CH3_PROXCG0                 0xA2  // Byte1
#define IQS269_MM_CH3_PROXCFG1                0xA3  // Byte0
#define IQS269_MM_CH3_ATI_TARGETS             0xA3  // Byte1
#define IQS269_MM_CH3_MULTIPLIERS             0xA4  // Byte0
#define IQS269_MM_CH3_PCC                     0xA4  // Byte1
#define IQS269_MM_CH3_THRESHOLD_P             0xA5  // Byte0
#define IQS269_MM_CH3_THRESHOLD_T             0xA5  // Byte1
#define IQS269_MM_CH3_THRESHOLD_DT            0xA6  // Byte0
#define IQS269_MM_CH3_HYSTERESIS              0xA6  // Byte1
#define IQS269_MM_CH3_FOLLOW_CHANNELS         0xA7  // Byte0
#define IQS269_MM_CH3_FOLLOW_WEIGHT           0xA7  // Byte1

/* Channel 4 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH4_SELECT_CRX              0xA8  // Byte0
#define IQS269_MM_CH4_SELECT_CTX              0xA8  // Byte1
#define IQS269_MM_CH4_PROXCTL                 0xA9  // Byte0
#define IQS269_MM_CH4_PROXCG0                 0xA9  // Byte1
#define IQS269_MM_CH4_PROXCFG1                0xAA  // Byte0
#define IQS269_MM_CH4_ATI_TARGETS             0xAA  // Byte1
#define IQS269_MM_CH4_MULTIPLIERS             0xAB  // Byte0
#define IQS269_MM_CH4_PCC                     0xAB  // Byte1
#define IQS269_MM_CH4_THRESHOLD_P             0xAC  // Byte0
#define IQS269_MM_CH4_THRESHOLD_T             0xAC  // Byte1
#define IQS269_MM_CH4_THRESHOLD_DT            0xAD  // Byte0
#define IQS269_MM_CH4_HYSTERESIS              0xAD  // Byte1
#define IQS269_MM_CH4_FOLLOW_CHANNELS         0xAE  // Byte0
#define IQS269_MM_CH4_FOLLOW_WEIGHT           0xAE  // Byte1

/* Channel 5 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH5_SELECT_CRX              0xAF  // Byte0
#define IQS269_MM_CH5_SELECT_CTX              0xAF  // Byte1
#define IQS269_MM_CH5_PROXCTL                 0xB0  // Byte0
#define IQS269_MM_CH5_PROXCG0                 0xB0  // Byte1
#define IQS269_MM_CH5_PROXCFG1                0xB1  // Byte0
#define IQS269_MM_CH5_ATI_TARGETS             0xB1  // Byte1
#define IQS269_MM_CH5_MULTIPLIERS             0xB2  // Byte0
#define IQS269_MM_CH5_PCC                     0xB2  // Byte1
#define IQS269_MM_CH5_THRESHOLD_P             0xB3  // Byte0
#define IQS269_MM_CH5_THRESHOLD_T             0xB3  // Byte1
#define IQS269_MM_CH5_THRESHOLD_DT            0xB4  // Byte0
#define IQS269_MM_CH5_HYSTERESIS              0xB4  // Byte1
#define IQS269_MM_CH5_FOLLOW_CHANNELS         0xB5  // Byte0
#define IQS269_MM_CH5_FOLLOW_WEIGHT           0xB5  // Byte1

/* Channel 6 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH6_SELECT_CRX              0xB6  // Byte0
#define IQS269_MM_CH6_SELECT_CTX              0xB6  // Byte1
#define IQS269_MM_CH6_PROXCTL                 0xB7  // Byte0
#define IQS269_MM_CH6_PROXCG0                 0xB7  // Byte1
#define IQS269_MM_CH6_PROXCFG1                0xB8  // Byte0
#define IQS269_MM_CH6_ATI_TARGETS             0xB8  // Byte1
#define IQS269_MM_CH6_MULTIPLIERS             0xB9  // Byte0
#define IQS269_MM_CH6_PCC                     0xB9  // Byte1
#define IQS269_MM_CH6_THRESHOLD_P             0xBA  // Byte0
#define IQS269_MM_CH6_THRESHOLD_T             0xBA  // Byte1
#define IQS269_MM_CH6_THRESHOLD_DT            0xBB  // Byte0
#define IQS269_MM_CH6_HYSTERESIS              0xBB  // Byte1
#define IQS269_MM_CH6_FOLLOW_CHANNELS         0xBC  // Byte0
#define IQS269_MM_CH6_FOLLOW_WEIGHT           0xBC  // Byte1

/* Channel 7 Settings - Read/Write = 14 bytes */
#define IQS269_MM_CH7_SELECT_CRX              0xBD  // Byte0
#define IQS269_MM_CH7_SELECT_CTX              0xBD  // Byte1
#define IQS269_MM_CH7_PROXCTL                 0xBE  // Byte0
#define IQS269_MM_CH7_PROXCG0                 0xBE  // Byte1
#define IQS269_MM_CH7_PROXCFG1                0xBF  // Byte0
#define IQS269_MM_CH7_ATI_TARGETS             0xBF  // Byte1
#define IQS269_MM_CH7_MULTIPLIERS             0xC0  // Byte0
#define IQS269_MM_CH7_PCC                     0xC0  // Byte1
#define IQS269_MM_CH7_THRESHOLD_P             0xC1  // Byte0
#define IQS269_MM_CH7_THRESHOLD_T             0xC1  // Byte1
#define IQS269_MM_CH7_THRESHOLD_DT            0xC2  // Byte0
#define IQS269_MM_CH7_HYSTERESIS              0xC2  // Byte1
#define IQS269_MM_CH7_FOLLOW_CHANNELS         0xC3  // Byte0
#define IQS269_MM_CH7_FOLLOW_WEIGHT           0xC3  // Byte1


/* I2C Control Settings - Read/Write = 2 bytes */
#define IQS269_MM_I2C_CONTROL_SETTINGS		  0xF2

/* HALL UI Enable */
#define IQS269_MM_HALL_UI_ENABLE			  0xF5

/** @}
  * End group IQS269 Memory Map
  */
  
#endif /* __IQS269_ADDRESSES_H */
