/**
	*****************************************************************************
	* @file			iqs269_typedefs.h
	* @ingroup	azq_drivers_iqs269
	* @brief 		Typedefs for easy handling of IQS269 settings and data.
  * @note     All selectable items are preceded with 'IQS269_' to prevent
  *           possible conflicts with items from other IQS devices.
	*****************************************************************************/
  
#ifndef __IQS269_TYPEDEFS_H
#define __IQS269_TYPEDEFS_H

/** @addtogroup azq_drivers_iqs269
  * @{
  */

/** @name   IQS269 Enumerations
  * @brief  Enums which facilitate selection of IQS269 functionality.
  * @{
  */

/** @brief  Enum for IQS269 channels. */
typedef enum{
  IQS269_CH0,
  IQS269_CH1,
  IQS269_CH2,
  IQS269_CH3,
  IQS269_CH4,
  IQS269_CH5,
  IQS269_CH6,
  IQS269_CH7
} IQS269_Channel_e;

/** @brief  Enum for IQS269 sliders. */
typedef enum{
  IQS269_SL0,
  IQS269_SL1
} IQS269_Slider_e;


/** @brief    Enum for IQS269 power modes. */
typedef enum{
  IQS269_POWER_MODE_NORMAL    = 0,
  IQS269_POWER_MODE_LOW       = 1,
  IQS269_POWER_MODE_ULTRA_LOW = 2,
  IQS269_POWER_MODE_HALT      = 3
} IQS269_PowerMode_e;

/** @brief  Enum for IQS269 system clock speeds. */
typedef enum {
  IQS269_SYSTEM_CLOCK_4MHZ,
  IQS269_SYSTEM_CLOCK_16MHZ
} IQS269_SystemClock_e;

/** @brief  Enum for IQS269 Abs Cap Filling Cap Selection. */
typedef enum {
  IQS269_0pF5,
  IQS269_1pF0,
  IQS269_1pF5,
  IQS269_2pF0
} IQS269_Fill_Cap_e;

/** @brief  Enum for IQS269 AbsCap Measurement Selection. */
typedef enum {
  IQS269_C0,
  IQS269_C1
} IQS269_Cap_Meas_e;

/** @brief Enum for I2C stop or restart */
typedef enum{
	I2C_STOP,
	I2C_RESTART
} IQS269_StopRestart_e;


typedef enum {
	FLICK_NEG_1 = (uint8_t) 0x00,
	FLICK_POS_1,
	HOLD_1,
	TAP_1,
	FLICK_NEG_0,
	FLICK_POS_0,
	HOLD_0,
	TAP_0,
	NO_GESTURE_0,
	NO_GESTURE_1,
	NO_SLIDER_SELECTED
} IQS269_gesture_e;


/** @brief  Enum for ATI modes.  */
typedef enum {
  IQS269_ATI_MODE_DISABLED     					= 0b00,
  IQS269_ATI_MODE_PARTIAL      					= 0b01,
  IQS269_ATI_MODE_SEMI_PARTIAL 					= 0b10,
  IQS269_ATI_MODE_FULL         					= 0b11
} IQS269_AtiMode_e;

/** @brief  Enum for IQS269 sensor modes.  */
typedef enum{
  IQS269_SENSOR_MODE_SURFACE            = 0x00,
  IQS269_SENSOR_MODE_PROJECTED          = 0x01,
  IQS269_SENSOR_MODE_DIFF_ST_MEMS       = 0x02,
  IQS269_SENSOR_MODE_DIFF_ST_MEMS_INV   = 0x03,
  IQS269_SENSOR_MODE_DIFF_AZQ_MEMS      = 0x04,
  IQS269_SENSOR_MODE_DIFF_AZQ_MEMS_INV  = 0x05,
  IQS269_SENSOR_MODE_INDUCTANCE_SELF    = 0x08,
  IQS269_SENSOR_MODE_INDUCTANCE_EXT     = 0x09,
  IQS269_SENSOR_MODE_INDUCTANCE_INT     = 0x0A,
  IQS269_SENSOR_MODE_EXTERNAL           = 0x0C,
  IQS269_SENSOR_MODE_ALS                = 0x0D,
  IQS269_SENSOR_MODE_HALL               = 0x0E,
  IQS269_SENSOR_MODE_TEMPERATURE        = 0x0F
}IQS269_SensorMode_e;

/** @} End group IQS269 Enumerations */

/** @} End group azq_drivers_iqs269 */

#endif	/* __IQS269_TYPEDEFS_H */
