/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    draw.c						 				  	 	  	  	  *
 * @brief 	    LCD driver file							  		  	  	  	  *
 * @author 		GF Visagie - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    13/02/2020                                                    *
 ******************************************************************************/


#ifndef __DRAW_H__
#define __DRAW_H__

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "fonts.h"
#include "Azoteq_logo_iqs269_array.h"

#define SSD1306_MIRROR_VERT
#define SSD1306_MIRROR_HORIZ 	//define to fix

// SSD1306 OLED height in pixels
#define SSD1306_HEIGHT          64

// SSD1306 width in pixels
#define SSD1306_WIDTH           128

// some LEDs don't display anything in first two columns
// #define SSD1306_WIDTH           130

// Enumeration for screen colors
typedef enum {
    Black = 0x00, // Black color, no pixel
    White = 0x01  // Pixel is set. Color depends on OLED
} SSD1306_COLOR;

#pragma pack(1)
// Struct to store transformations
typedef struct {
    uint16_t CurrentX;
    uint16_t CurrentY;
    uint8_t Inverted;
    uint8_t Initialized;
    uint8_t State;
    uint8_t init_state;
    uint8_t IF_State;
    uint8_t update_state;
} SSD1306_t;
#pragma pack(4)

/**
* @brief  LCD state
*/
typedef enum {
	LCD_NONE = (uint8_t) 0x00,			// 0x00
	LCD_INIT,
	LCD_RUN,
	LCD_DONE
} LCD_e;

/**
* @brief  LCD Interface state
*/
typedef enum {
	LCD_IF_NONE = (uint8_t) 0x00,			// 0x00
	LCD_IF_IDLE,
	LCD_IF_BUSY,
	LCD_IF_READY
} LCD_IF_e;

/**
* @brief  LCD update state
*/
typedef enum {
	LCD_UPDATE_NONE = (uint8_t) 0x00,			// 0x00
	LCD_UPDATE_IDLE,
	LCD_UPDATE_START,
	LCD_UPDATE_COMMAND,
	LCD_UPDATE_DATA,
	LCD_UPDATE_DONE
} LCD_update_e;

/**
* @brief  LCD init state
*/
typedef enum {
	LCD_INIT_NONE = (uint8_t) 0x00,			// 0x00
	LCD_INIT_RESET,
	LCD_INIT_WRITE_SETTINGS,
	LCD_INIT_BUSY,
	LCD_INIT_CLEAR_SCREEN,
	LCD_INIT_FINAL,
	LCD_INIT_DONE
} LCD_init_e;

// Procedure definitions
void run_ssd1306_Init(void);
void ssd1306_Fill(SSD1306_COLOR color);
void ssd1306_Clear_region(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, SSD1306_COLOR color);	// TODO: Alwino toets funksie
void ssd1306_LoadLogo(void);
bool ssd1306_UpdateScreen(void);
void ssd1306_DrawPixel(uint8_t x, uint8_t y, SSD1306_COLOR color);
char ssd1306_WriteChar(char ch, FontDef Font, SSD1306_COLOR color);
char ssd1306_WriteString(char* str, FontDef Font, SSD1306_COLOR color);
void ssd1306_SetCursor(uint8_t x, uint8_t y);
void SSD1306_DrawLine(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, SSD1306_COLOR c);
void SSD1306_DrawRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1306_COLOR c);
void SSD1306_DrawFilledRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1306_COLOR c);
void SSD1306_DrawTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1306_COLOR color);
void SSD1306_DrawFilledTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1306_COLOR color);
void SSD1306_DrawCircle(int16_t x0, int16_t y0, int16_t r, SSD1306_COLOR c);
void SSD1306_DrawFilledCircle(int16_t x0, int16_t y0, int16_t r, SSD1306_COLOR c);
void drawCircleHelper( int16_t x0, int16_t y0, int16_t r, uint8_t cornername, SSD1306_COLOR color);
void fillCircleHelper(int16_t x0, int16_t y0, int16_t r, uint8_t cornername, int16_t delta, SSD1306_COLOR color);
void drawRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, SSD1306_COLOR color);
void fillRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, SSD1306_COLOR color);

void run_LCD_Interface(void);
void run_LCD(void);
bool ssd1306_CheckScreenReady(uint32_t displayTime);

// Low-level procedures
void ssd1306_Reset(void);
bool ssd1306_WriteCommand(uint8_t byte);
bool ssd1306_WriteData(uint8_t* buffer, size_t buff_size);
bool ssd1306_WriteCommandBuffer(uint8_t* buffer, size_t buff_size);

void ssd1306_Init(SSD1306_t *ssd1306);

#endif // __DRAW_H__
