/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    spi_dma.h						 				  	 	  	  *
 * @brief 	    spi dma driver header file					  		  	  	  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    06/02/2019                                                    *
 ******************************************************************************/

#ifndef __SPI_DMA_H__
#define __SPI_DMA_H__

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include <stdbool.h>
/* Exported types ------------------------------------------------------------*/
/**
 * @brief  SPI State Defintion
 */
typedef enum
{
	SPI_NONE     				= 	0x00U,
	SPI_IDLE		  			= 	0x01U,
	SPI_WAIT_FOR_TX_DMA			= 	0x02U,
	SPI_WAIT_FOR_RX_DMA		  	= 	0x03U,
	SPI_DONE		  			= 	0x04U
} SPI_State_e;

/* Exported constants --------------------------------------------------------*/

/* SPI Timeout */
#define		SPI_TIMEOUT			60000

/* Define NSS CS Pin */
#define		SPI_NSS_CS_PORT		GPIOB
#define 	SPI_NSS_CS_PIN		LL_GPIO_PIN_0	//PB0

/* Define SCK Pin */
#define		SPI_SCK_PORT		GPIOA
#define 	SPI_SCK_PIN			LL_GPIO_PIN_5	//PA5

/* Define MISO Pin */
#define 	SPI_MISO_PORT		GPIOA
#define 	SPI_MISO_PIN		LL_GPIO_PIN_6	//PA6

/* Define MOSI Pin */
#define 	SPI_MOSI_PORT		GPIOA
#define 	SPI_MOSI_PIN		LL_GPIO_PIN_7	//PA7

/* Define RES Pin */
#define 	SPI_RES_PORT		GPIOA
#define 	SPI_RES_PIN			LL_GPIO_PIN_8	//PA8

/* Define DC Pin */
#define 	SPI_DC_PORT			GPIOA
#define 	SPI_DC_PIN			LL_GPIO_PIN_9	//PA9

/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
void init_spi(void);
bool is_spi_busy(void);
void DMA1_ReceiveComplete_Callback(void);
void DMA1_TransmitComplete_Callback(void);
void SPI1_TransferError_Callback(void);
void set_SPI_CS_NSS_pin (void);
void reset_SPI_CS_NSS_pin (void);
void spi_readwrite(uint16_t TX_DataLength, uint8_t *TX_DataBuffer, uint16_t RX_DataLength, uint8_t *RX_DataBuffer);


#endif /* __SPI_DMA_H__ */
