/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    List.h							 				  	 	      *
 * @brief 	    Generic C list									  		  	  *
 * @author 		AJ van der Merwe - Azoteq PTY Ltd                             *
 * @version 	V1.0.0                                                    	  *
 * @date 	    27/03/2017                                                    *
 ******************************************************************************/

#ifndef __LIST_H__
#define __LIST_H__

/* Includes ------------------------------------------------------------------*/
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

/** Typedef for generic node */
typedef struct Node_s Node_t;

/** Typedef for List type */

#pragma pack(1)
typedef struct {
    uint8_t listSize;       // List size is limited to 255
    uint8_t memorySize;     // Size of the memory - remember that it is not ! size_t !
    uint8_t maxListSize;    //  Check that the size is smaller than this number, otherwise, shift head node out
    Node_t *pHead;          // Pointer to the head node
    Node_t *pTail;          // Pointer to the tail node - there are suggestions about chucking this out?
}List_t;
#pragma pack(4)

void List_init(List_t *_this, uint8_t memorySize, uint8_t maxSize);
bool List_insertStart(List_t *_this, uint8_t key, const void *data);
bool List_insertEnd(List_t *_this, uint8_t key, const void *data);
void List_deleteFirst(List_t *_this);
void List_deleteLast(List_t *_this);
bool List_deleteItem(List_t *_this, uint8_t index);
bool List_deleteItemwithKey(List_t *_this, uint8_t key);
void List_empty(List_t *_this);
// We should actually delete the list as well - recapture the memory
void List_getLast(const List_t *_this, void *data);
bool List_getItem(List_t *_this, uint8_t index, void *data);
bool List_editItem(List_t *_this, uint8_t index, void *data);
int8_t List_keyExists(List_t *_this, uint8_t key, void *data);
uint8_t List_getSize(const List_t *_this);
bool List_isEmpty(const List_t *_this);
// We can even implement search and sort

#endif  /* __LIST_H__ */
