/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    AZQ_util.h						  	 				  	 	  *
 * @brief 	    Utilities for Azoteq Luminaire				  		  	  	  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    23/08/2018                                                    *
 ******************************************************************************/

#ifndef __AZQ_UTIL_H__
#define __AZO_UTIL_H__

/* Includes ------------------------------------------------------------------*/

#include "main.h"
#include <stdbool.h>
#include <string.h>
#include <stdio.h>

//Create a typedef for radar use
typedef int64_t AzoVar_t;
#define AZOVAR_SHIFT	16
#define cast_to_AzoVar(A) ((AzoVar_t) (A<<AZOVAR_SHIFT))
//#define cast_ALS_to_AzoVar(A) ( (((AzoVar_t)A)<<(AZOVAR_SHIFT-ALS_SHIFT)))


#define AZO_1 (1<<AZOVAR_SHIFT)

#define AzoVar_to_uint32_t(A) (uint32_t)((A)>>(AZOVAR_SHIFT-ALS_SHIFT))
// Break a large number into smaller pieces


#define GET_LARGE_NUMBER_7(X)		(uint8_t)((X>>56)&0xFF)
#define GET_LARGE_NUMBER_6(X)		(uint8_t)((X>>48)&0xFF)
#define GET_LARGE_NUMBER_5(X)		(uint8_t)((X>>40)&0xFF)
#define GET_LARGE_NUMBER_4(X)		(uint8_t)((X>>32)&0xFF)
#define GET_LARGE_NUMBER_3(X)		(uint8_t)((X>>24)&0xFF)
#define GET_LARGE_NUMBER_2(X)		(uint8_t)((X>>16)&0xFF)
#define GET_LARGE_NUMBER_1(X)		(uint8_t)((X>>8)&0xFF)
#define GET_LARGE_NUMBER_0(X)		(uint8_t)(X&0xFF)

// Make a larger number
//#define SET_LARGE_NUMBER_64(A,B,C,D,E,F,G,H)		(uint64_t)((0xFF00000000000000&(uint64_t)(A<<56)) | (0x00FF000000000000&(uint64_t)(B<<48))
//															| (0x0000FF0000000000&(uint64_t)(C<<40)) | (0xFF00000000&(uint64_t)(D<<32))
//															| (0xFF000000&(uint64_t)(E<<24)) | (0x00FF0000&(uint64_t)(F<<16))
//															| (0x0000FF00&(uint64_t)(G<<8)) | (0xFF&(uint64_t)(H)))

#define SET_LARGE_NUMBER_64(A,B)							(uint64_t)((0xFFFFFFFF00000000&(uint64_t)((uint64_t)A<<32)) | (0x00000000FFFFFFFF&(uint64_t)((uint64_t)B)))

// Make a larger number
#define SET_LARGE_NUMBER_32(A,B,C,D)		(uint32_t)((0xFF000000&(uint32_t)(A<<24)) | (0x00FF0000&(uint32_t)(B<<16)) \
		| (0x0000FF00&(uint32_t)(C<<8)) | (0xFF&(uint32_t)(D)))

// Make a larger number
#define SET_LARGE_NUMBER_16(A,B)		(uint16_t)((0x0000FF00&(uint16_t)(A<<8)) | (0xFF&(uint16_t)(B)))


/* BETA filters */

#define BETA_VALUE_FASTER					1
#define BETA_VALUE_FAST						2
#define BETA_VALUE_SLOW						3
#define BETA_VALUE_VERY_SLOW				5


#define	FUNCTION_NOT_USED 					0

#define AZQ_BETA_FILT(X, Y, B) (((X)>>(B)) + (Y) - ((Y)>>(B)))

#define 					IS_I2C1(X) (((X)==I2C1_BASE))
#define 					IS_I2C2(X) (((X)==I2C2_BASE))
#define 					IS_NOT_I2C(X) (((X)!=I2C1_BASE) && ((X)!=I2C2_BASE))
// Function to generate index for I2C_Com_Variable array according to I2C peripheral selected.
#define 					RETURN_I2C_ENUM(X) (0*(IS_I2C1((uint32_t)X)) + 1*(IS_I2C2((uint32_t)X) + 2*IS_NOT_I2C((uint32_t)X)))

#define						IS_GPIOA(X) (((X)==GPIOA_BASE))
#define						IS_GPIOB(X) (((X)==GPIOB_BASE))
#define						IS_GPIOC(X) (((X)==GPIOC_BASE))
#define						IS_GPIOD(X) (((X)==GPIOD_BASE))
#define 					IS_NOT_GPIO(X) (((X)!=GPIOA_BASE) && ((X)!=GPIOB_BASE) && ((X)!=GPIOC_BASE) && ((X)!=GPIOD_BASE))
#define 					RETURN_GPIO_ENUM(X) (0*(IS_GPIOA((uint32_t)X)) + 1*(IS_GPIOB((uint32_t)X)) + 2*(IS_GPIOC((uint32_t)X)) + 3*(IS_GPIOD((uint32_t)X)) + 4*(IS_NOT_GPIO((uint32_t)X)))


/* Public Functions ------------------------------------------------------------*/
uint32_t HAL_GetTick(void);
bool HAL_CheckTick(uint32_t timestamp, uint32_t compare_value, bool larger_check);
void print_int(uint16_t value, char* int_char_buffer);
void Enable_GPIO_Clock(GPIO_TypeDef *gpiox);
void Enable_I2C_Clock(I2C_TypeDef *I2Cx);
void init_I2C_interrupt(I2C_TypeDef *I2Cx, uint32_t priority);
void gpio_init(GPIO_TypeDef *gpio, uint32_t pin, uint32_t mode, uint32_t af_function, uint32_t type, uint32_t pu_pd, uint32_t speed);
void AZQ_update_msTick(void);
void Configure_TIM7(void);
void Configure_EXTI(GPIO_TypeDef *gpiox, uint32_t IR_Line, uint32_t IR_priority, bool rising, bool falling, void *_exti_callback);
void checkInterrupt_Line(uint32_t IR_Line);
uint8_t Get_IR_Index(uint32_t IR_Line);

#endif /* __AZQ_UTIL_H__ */
